/** @file UnityMainRunner.c
 *
 * Automatische Modultests
 *
 * $Date: 2020-08-13 11:28:54 +0200 (Do, 13 Aug 2020) $
 * $Revision: 68 $
 *
 * @date 2.02.2018 */



#define RUN_TEST(TestFunc, TestLineNum) \
{ \
  Unity.CurrentTestName = #TestFunc; \
  Unity.CurrentTestLineNumber = TestLineNum; \
  Unity.NumberOfTests++; \
  if (TEST_PROTECT()) \
  { \
      setUp(); \
      TestFunc(); \
  } \
  if (TEST_PROTECT()) \
  { \
    tearDown(); \
  } \
  UnityConcludeTest(); \
}

#include "unity.h"
#include <setjmp.h>
#include <stdio.h>



/*=======External Functions This Runner Calls=====*/
extern void setUp(void);
extern void tearDown(void);


/*=======Test Reset Option=====*/
void resetTest(void);
void resetTest(void)
{
  tearDown();
  setUp();
}


void setUp(void) {}
void tearDown(void) {}


void Test_fail(void)
{
    TEST_ASSERT_EQUAL_HEX( 0x1, 0x2 );
}




#include "unity_fixture.h"


/** Automatische Modultests: Test runner */
void RunAllTests(void)
{
    #ifdef STM32F10x
        /* Der STM32F103RB hat nicht genuegend Flash, um sowohl die Modultests als auch
           die normalen Funktionen zu halten. */
        return;
    #endif


    RUN_TEST_GROUP(bitmath);
    RUN_TEST_GROUP(color);
    RUN_TEST_GROUP(crc);
    RUN_TEST_GROUP(delay);
    RUN_TEST_GROUP(font);
    RUN_TEST_GROUP(fontU8g2);
    RUN_TEST_GROUP(gfx_color_circles_quadrant);
    RUN_TEST_GROUP(intmath);
    RUN_TEST_GROUP(menu_color);
    RUN_TEST_GROUP(myassert);
    RUN_TEST_GROUP(mystring);
    RUN_TEST_GROUP(pushbutton_time);
    RUN_TEST_GROUP(rect);
    RUN_TEST_GROUP(strfun);
    RUN_TEST_GROUP(text_field);
    RUN_TEST_GROUP(text_window);
    RUN_TEST_GROUP(tft_text);
    RUN_TEST_GROUP(wallclock);
}



#include <stdbool.h>
#include "text_window.h"
#include "text_cursor.h"
#include "sw_config.h"
#include "userinterface.h"
#include "u8g2_fontdata.h"





#if defined( STM32F10x ) || defined( STM32F4XX ) || DOXYGEN

    /** Auf der Zielplattform wird ein neues Fenster aufgemacht (Konsolenschriftart) und am Ende
     * nach den Modultests auf einen Tastendruck gewartet. */
    int runUnityUnittests(void)
    {
        int retval;

        int argc = 0;
        const char *argv[1] =  {NULL};

        Ui_t Uim, *Ui = &Uim;
        ui_init(Ui, "Test delay_ns()");
        ui_clear();


        textwindow_setFont(font_monospaceTerminal);
        textwindow_setColors( (Colors_t) {green, black} );
        cursor_setPosition((Coord2_t) {0, 0});

        retval = UnityMain(argc, argv, RunAllTests);


        ui_printf("          ");
        ui_deinit(Ui);

        while( !ui_getAction() );
        return retval;
    }


#elif defined( TARGET_MOCKUP_SDL ) || defined( TARGET_UNIT_TEST_PC )

    /** Im SDL-Mockup und bei den PC-Tests werden die Ergebnisse der Modultests in der Konsole
     * ausgegeben, deswegen kann der Detailgrad hoeher sein. */
    int runUnityUnittests(void)
    {
        int argc = 2;
        const char *argv[3] = {"platform", "-v", NULL};

        return UnityMain(argc, argv, RunAllTests);
    }

#else
    #error "unknown platform"

#endif


