/** @file debughelp.c
 *
 * Hilfs-Konstruktionen zum Debugging und Profiling
 *
 * Die Debug-Level kommen hauptsaechlich in Fontdarstellung und Textboxen zum Einsatz.
 *
 * $Date: 2020-08-12 09:22:52 +0200 (Mi, 12 Aug 2020) $
 * $Revision: 60 $
 *
 * @date 14.03.2018 @author Nicolas */


#include "debughelp.h"
#include "nstdlib.h"
#include <string.h>
#include <stdbool.h>
#include "gfx_draw_rectangles.h"



/** Hilfsvariable: Aktuell einstellter Debug-Darstellungs-Level */
enum debugoutput_e debugOutput = debugoutput_none;



/** Debug-Ausgabe setzen
 *
 * @param[in] c: Zusaetzlicher Debug-Level */
void debugoutput_enable(enum debugoutput_e c)
{
    debugOutput |= c;
}



/** Debug-Ausgabe entfernen
 *
 * @param[in] c: Zu entfernender Debug-Level */
void debugoutput_disable(enum debugoutput_e c)
{
    debugOutput &= ~c;
}



/** Debug-Ausgabe abfragen
 *
 * @param[in] c: Zu entfernender Debug-Level */
ufast8_t debugoutput_getstatus(void)
{
    return debugOutput;
}



/** Debug-Hilfsfunktion: Bounding-Box anzeigen.
 *
 * @param[in] BB: Rechteck, Bounding-Box
 * @param[in] color: Linienfarbe */
void debugoutput_showBox(Rect_t BB, color_t color)
{
    if( debugOutput & debugoutput_boxes )
    {
        drawRect(BB, (Colors_t) {color, transparent});
    }
}
