/** @file hw_config.h
 *
 * Einstellungen bezueglich Hardware und Peripherie, insbesondere die I/O-Ressourcen des gesamten
 * Projekts
 *
 * $Date: 2020-08-12 18:19:08 +0200 (Mi, 12 Aug 2020) $
 * $Revision: 67 $
 *
 * @date 01.07.2020 @author Nicolas */


#ifndef HW_CONFIG_H_INCLUDED
    #define HW_CONFIG_H_INCLUDED

    #if defined( TARGET_HARDWARE_STEPCNT_REV01 )
        #define GPIO_HARDWARE

        #define GLCD_PCD8544
        #define GLCD_FRAMEBUFFER

        #define GLCD_PCD8544_SCLK_PIN   PA5
        #define GLCD_PCD8544_SDIN_PIN   PA7
        #define GLCD_PCD8544_DC_PIN     PA2
        #define GLCD_PCD8544_CE_PIN     PA1
        #define GLCD_PCD8544_RST_PIN    PA0

        #define I2C_BITBANG
        #define I2C_SDA_PIN             PA3                   /**< I2C-Pin: SDA (Data) */
        #define I2C_SCL_PIN             PA4                   /**< I2C-Pin: SCL (Clock) */

    #elif defined( TARGET_UNIT_TEST_PC )
        #define GLCD_NONE
        #define I2C_MOCK 1
        #define I2C_SDA_PIN             PA3                   /**< I2C-Pin: SDA (Data) */
        #define I2C_SCL_PIN             PA4                   /**< I2C-Pin: SCL (Clock) */

    #elif defined( TARGET_MOCKUP_SDL )
        #define GLCD_EMULATION_SCALING 3
        #define GLCD_EMULATION_84x48
        #define GLCD_FRAMEBUFFER
        #define I2C_MOCK 1
        #define I2C_SDA_PIN             PA3                   /**< I2C-Pin: SDA (Data) */
        #define I2C_SCL_PIN             PA4                   /**< I2C-Pin: SCL (Clock) */

    #else
        #error "Invalid Target"

    #endif



    /* Sampling-Pins */
    #define SAMPLE_GPIO GPIOB
    #define SAMPLE_PIN_STEP_PLUS  GPIO_Pin_15
    #define SAMPLE_PIN_STEP_MINUS GPIO_Pin_14
    #define SAMPLE_PIN_DIR_PLUS  GPIO_Pin_5
    #define SAMPLE_PIN_DIR_MINUS GPIO_Pin_4


    /** @name I2C
     *
     * @{ */

    /* Timeouts fuer jede I2C-Operation in Millisekunden */
    #define I2C_NORMALTIMEOUT_MS 200
    #define I2C_BUSYTIMEOUT_MS 5000
    #define I2C_RESETTIMEOUT_MS 800


	/** Freitakten / I2C-Bus-Reset:  10 s ergibt 50 kHz */
    #define I2C_BITBANGING_DELAY_US 10


    /** Haupt-I2 (z.B.\ fuer Bus-Scan) - pro forma, da ohnehin Bitbanging */
    #define I2C_MAIN_I2Cx I2C1
    /** @} */



    /** @name EEPROM
     * EEPROM C24x, angebunden ueber I2C
     * @{ */
    #define EEPROM_C24X_I2C_DEVICE I2C1         /**<  I2C1|I2C2 */
    #define EEPROM_C24X_I2C_ADDRESS 0xA0        /**<  I2C-Adresse (linksbuendig) */
    #define EEPROM_C24X_SIZE_BYTES 256          /**<  EEPROM-Groesse in BYTES */
    #define EEPROM_C24X_N_ADDRESS_BYTES 1       /**<  1|2 Interne Adressierung: Anzahl der Adress-
                                                      Bytes,  24LC02: 1, 24LC64: 2 */
    #define EEPROM_C24X_WRITE_CYCLE_TIME_MS 5   /**< Wartezeit fuer Schreibzyklus [Millisekunden] */
    /** @} */


	/** @name Drehgeber zur Benutzereingabe
	 * @{ */
	#define ENC0_B_PIN PB10
	#define ENC0_A_PIN PB2
    #define ENC_N 1
    /** @} */



    /*** Enter-Taster
     * Der Drehgeber-Taster auf Zeit abgefragt */
    #define BUTTONTIME0_PIN   PB11




    /** @name Not present - Nicht-vorhandene Hardware
     *
     * @{ */
    #define TFT_ILI9XXX_CONNEXION_NONE
    #define I2C1_CLOCK 0
    #define I2C2_CLOCK 0

    /** Anzahl der zu entprellenden Taster und Schalter.
        Der einzige vorhandene Taster wird auf Zeit, nicht auf Druck abgefragt */
    #define N_SWITCH 0

    /** Rote LED zur Unterstuetzung der Morse-Ausgabe */
    #define LED_RT_PIN  NO_PIN

    /** ADC */
    #define MAIN_VOLTAGE_ADC_CHANNEL adc_none

    /** SPARE-Pins sind noch in einigen Tests drin - keiner der Pins ist auf Pads gefuehrt. */
	#define SPARE1_PIN   PB15
    #define SPARE0_PIN   PB3
    /** @} */


#endif
