/** @file settings.c
 *
 * @brief Harte und EEPROM-Einstellungen (Gesamtsystem)
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 01.03.2014 @author Nicolas */


#include "settings.h"
#include "nvram.h"
#include <string.h>
#include <stdlib.h>


#define EEPROM_SETTING_ADDRESS 0
#define SIZE_SETDATA (sizeof(Settings_t) )

/**
 * @addtogroup myStepCount_system
 * @{
 */


/** Konfigurationsdaten aus EEPROM auslesen und bei OK das globale struct ueberschreiben
 *
 * Im Fehlerfall benutzerinteraktive Funktion */
uint_fast8_t settings_read(void)
{
    uint8_t buffer[SIZE_SETDATA];
    uint_fast8_t status;
    status = nvram_read(buffer, EEPROM_SETTING_ADDRESS, SIZE_SETDATA);

    if( status==EXIT_SUCCESS )
    {
    	memcpy(&Settings, buffer, SIZE_SETDATA);
    	return EXIT_SUCCESS;
    }
    else
    {
        return status;
    }
}



/** Konfigurationsdaten in EEPROM schreiben.
 * Wenn das schreiben fehlerhaft war, sind die alten Konfigurationsdaten im EEPROM leider
 * verloren. */
uint_fast8_t settings_write(void)
{
    return nvram_write((uint8_t *) &Settings, EEPROM_SETTING_ADDRESS, SIZE_SETDATA);
}



/** Konfigurationsdaten auf default-Werte setzen.
 * z.B. fuer leeres EEPROM oder globalen Reset der Parameter */
void settings_default(void)
{
    Settings = (Settings_t) SETTINGS_DEFAULT0;
}


/** @} */
