/** @file settings.h
 *
 * @brief Harte und EEPROM-Einstellungen (Gesamtsystem)
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 01.08.2020 @author Nicolas */

#ifndef SETTINGS_H_INCLUDED
    #define SETTINGS_H_INCLUDED


    #define SETTINGS_MAGIC_VALUE 0xCAFE

    #include <stdbool.h>
    #include <stdint.h>
//    #include "frac.h"
    #include "sw_config.h"



    /** \struct Settings_s
     * @brief Alle nutzerseitig aenderbare Einstellungen
     *
     * Alle nutzerseitig zur Laufzeit aenderbaren Einstellungen befinden sich in diesem Struct,
     * und werden ins EEPROM geschrieben. Allerding sind nicht alle Einstellungen nutzerseitig
     * aenderbar */
    typedef struct Settings_s
    {
        int16_t spmmExp;     /**< Steps per Millimeter = 2^e*z/n */
        int16_t spmmNom;
        int16_t spmmDenom;
        /** Magischer Wert, um sicherzustellen, dass Settings-Variable ueberhaupt initialisiert ist.
         */
        uint16_t magic;
    }
    Settings_t;


    extern Settings_t Settings;


    /** Zusammenstellung der Default-Einstellungen */
    #define SETTINGS_DEFAULT0 \
    {\
        .spmmExp = 6,\
        .spmmNom = 125,\
        .spmmDenom = 9,\
        .magic = SETTINGS_MAGIC_VALUE,\
    }



	/* Funktionen */
	uint_fast8_t settings_read(void);
	uint_fast8_t settings_write(void);
	void settings_default(void);


#endif /* SETTINGS_H_INCLUDED */
