/** @file menues.c
 *
 * Einstellungsmenue
 *
 * $Date: 2020-08-12 14:11:34 +0200 (Mi, 12 Aug 2020) $
 * $Revision: 63 $
 *
 * @date 9.02.2014 @author Nicolas */


#include "menues.h"
#include "menu_color.h"
#include "menue_templates.h"
#include "settings.h"
#include "u8g2_fontdata.h"
#include "locale.h"
#include "ui_nvram.h"



Settings_t Settings;

constflash char MT_HEADLINE_SETTING[]  = "Flanken/mm";
constflash char MT_SETTING_EXP[]       = "Exp\t:\t 2^%i ";
constflash char MT_SETTING_NOM[]       = "Zhler\t:\t *%i ";
constflash char MT_SETTING_DENOM[]     = "Nenner\t:\t /%i ";



/* ********************************************************************************************** */
/*     Einstellungen-Men                                                                         */
/* ********************************************************************************************** */


/** Menueeintraege (Einstellungen) */
constflash Menuitem_t MenuSettingsData[] =
{
    /* Bei den obersten drei Eintraegen ist der Rueckgabewert interessant: */
    { MT_ABORT,                     .All     = {exitMenu}                                                      },
    { MT_RETURN_NOT_SAVE,           .All     = {exitMenu}                                                      },
    { MT_SAVEANDRET,                .All     = {exitMenu}                                                      },

    /* Werte */
    { MT_SETTING_EXP,               .Int16   = {int16,      &(Settings.spmmExp),    .min=0, .max=32,  .inc=1 } },
    { MT_SETTING_NOM,               .Int16   = {int16,      &(Settings.spmmNom),    .min=1, .max=500, .inc=1 } },
    { MT_SETTING_DENOM,             .Int16   = {int16,      &(Settings.spmmDenom),  .min=1, .max=500, .inc=1 } },
    { MT_NULL,                      .All     = {endmarker}                                                     },
};



/** Einstellungen: Kopfzeile/Kopfzeilenfunktion */
constflash Menuconfig_t MenuSettingsConfig =
{
    .Dimension = &Menudimension_nokiaLcd,
    .Color     = &ColorScheme_Monochrome,
    .Control   = &Menucontrol_standard,
    .headtext  = MT_HEADLINE_SETTING,
    .iconLeft  = 67, /* icon_reload, */
    .iconRight = 67, /* icon_reload, */
    .iconFont  = u8g2_font_open_iconic_check_1x_t,
};



/** Einstellungsmenue (interaktiv)
 * aendert die globale Variable Settings (siehe @link Settings_s @endlink und schreibt auf Wunsch
 * die Aenderungen in den nichtfluechtigen Speicher. */
void menuSettings(void)
{
    ui_read_eeprom();

    /* Sicherheitskopie der aktuellen Einstellungen */
    Settings_t Temp = Settings;

    uint8_t itemNo = menu_standard(MenuSettingsData, &MenuSettingsConfig, 0);

    switch( itemNo )
    {
        case 0:
            /* Abbrechen */
            Settings = Temp;
            break;

        case 1:
            /* Mit neuen Werten fortsetzen ohne zu speichern */
            break;

        case 2:
            /* Speichern & Ende */
            ui_write_eeprom();
            break;

        default:
            error("invalid return from menue");
    }
}
