﻿/** @file popups.c
 *
 * Popups/Messageboxen fuer alle Lebenslagen des Projekts (allgemein)
 *
 * $Date: 2020-08-04 11:16:17 +0200 (Di, 04 Aug 2020) $
 * $Revision: 38 $
 *
 * @date 26.02.2016 @author Nicolas */


#include "popups.h"
#include "messagebox.h"
#include <stddef.h>
#include "locale.h"


/** Fehlermeldung: "EEPROM-Daten nicht korrekt geschrieben" - muss mit einem Tastendruck quittiert
 * werden
 *
 * @return 0 */
uint_fast8_t popup_eepromWriteFailed(void)
{
    Usermessage_t Message;
    usermessageitem_t items[] = {MT_RETRY, MT_ABORT, MT_NULL};
    Message.type = messagetype_error;
    Message.text = LT_EEPROMWRITEFAILED;
    Message.items = items;
    Message.errcode = 0;
    return messagebox(&Message);
}



/** Erfolgsmeldung: "EEPROM-Daten korrekt geschrieben - muss mit einem Tastendruck quittiert werden
 *
 * @return 0 */
uint_fast8_t popup_eepromWriteSuccess(void)
{
    Usermessage_t Message;
    usermessageitem_t items[] = {MT_CONTINUE, MT_NULL};
    Message.type = messagetype_success;
    Message.text = LT_EEPROMWRITESUCCESS;
    Message.items = items;
    Message.errcode = 0;
    return messagebox(&Message);
}



/** Fehlermeldung: "EEPROM-Daten nicht korrekt gelesen - Nochmal, Abbrechen oder Defaultwerte
 * nutzen"
 *
 * @param[in] errcode: Fehlercode
 * @return Auswahl */
uint_fast8_t popup_eepromReadFailed(uint16_t errcode)
{
    Usermessage_t Message;
    usermessageitem_t items[] = {MT_RETRY, MT_USEDEFAULT, MT_NULL};
    Message.type = messagetype_error;
    Message.text = LT_EEPROMREADFAILED;
    Message.items = items;
    Message.errcode = errcode;
    return messagebox(&Message);
}
