/** @file ui_nvram.c
 *
 * User Interface: EEPROM lesen und schreiben und moegliche Benutzerinteraktion bei Fehlern
 *
 * $Date: 2020-08-01 20:41:52 +0200 (Sa, 01 Aug 2020) $
 * $Revision: 31 $
 *
 * @date Created: 23.03.2016 @author Nicolas */


#include "ui_nvram.h"
#include "settings.h"
#include "popups.h"
#include "myassert.h"



/** User Interface: EEPROM lesen, Benutzerinteraktion bei Fehlern */
void ui_read_eeprom(void)
{
    uint16_t status;

    do
    {
        status = settings_read();
        if( status != 0 )
        {
            /* Wenn Lesen fehlschlaegt, Benutzer fragen */
            switch( popup_eepromReadFailed(status) )
            {
                case 0:
                    /* nochmal versuchen */
                    break;

                case 1:
                    /* Defaultwerte */
                    settings_default();
                    status = 0;
                    break;

                default:
                    error("invalid return value");

            }
        }
    } while( status != 0 );
}



/** User Interface: EEPROM schreiben, Erfolgsmeldung oder Benutzerinteraktion bei Fehlern */
void ui_write_eeprom(void)
{
    uint_fast16_t status;

    do
    {
        status = settings_write();
        if( status != 0 )
        {
            /* Wenn Schreiben fehlschlaegt, Benutzer fragen: Nochmal/Cancel */
            switch( popup_eepromWriteFailed() )
            {
                case 0:
                    /* Nochmal versuchen */
                    break;

                case 1:
                    /* Abbrechen */
                    settings_default();
                    status = 0;
                    break;

                default:
                    error("invalid return value");
            }
        }
        else
        {
            /* Popup: Schreiben erfolgreich. Muss nur mit Tastendruck quittiert werden. */
            popup_eepromWriteSuccess();
        }
    } while( status != 0 );
}
