/** @file menue_templates.c
 *
 * Vorlagen fuer unterschiedliche Menues und ihre Darstellung
 *
 *  * Farbwahl
 *  * Steuerung
 *  * Startfunktion
 *  * Schleifenfunktion
 *
 * $Date: 2020-08-03 13:10:32 +0200 (Mo, 03 Aug 2020) $
 * $Revision: 1261 $
 *
 * @date 12.03.2018 @author Nicolas */


#include "menue_templates.h"
#include "messagebox.h"
#include "sw_config.h"
#include "locale.h"
#include "menu_color.h"
#include "gfx_draw.h"
#include "glcd_generic.h"
#include "u8g2_fontdata_icons.h"
#include "userinterface.h"



/** Menue-Abmessungen (Template fuer Messagebox) */
constflash Menudimension_t Menudimension_messagebox_nokia5510 =
{
    .Area = {0, 0, GFX_MAX_WIDTH_PX, GFX_MAX_HEIGHT_PX+5}, /* Ueberhoehe wird von Text-Clipping-Funktionen abgefangen */
    .headheight = 2*GFX_MAX_HEIGHT_PX/3,
    .scrollbarWidth = 0,
    .margin = 0,
    .textIndent = 0,
    .cornerRadius = 0,
    .headsep = -4,
};



/** Messagebox: Piktogramme zusammengestellt fuer Font u8g2_font_open_iconic_check_1x_t */
constflash Usermessage_Type_t Usermessage_Types[] =
{
    [messagetype_info]    = {.icon=67, .iconColor=0 },
    [messagetype_error]   = {.icon=68, .iconColor=0},
    [messagetype_success] = {.icon=64, .iconColor=0},
    [messagetype_end]     = {.icon=0,  .iconColor=0},
};



/** Messagebox: Default-Einstellungen
 * Piktogramme und Kopfzeilentext werden zur Laufzeit ausgetauscht. */
constflash Menuconfig_t MenuConfig_Messagebox_default =
{
    .Dimension  = &Menudimension_messagebox_nokia5510,
    .Color      = &ColorScheme_Monochrome,
    .Control    = &Menucontrol_standard,
    .headtext   = NULL,
    .iconLeft   = 0,
    .iconRight  = 0,
    .iconFont   = u8g2_font_open_iconic_check_1x_t,
};




/** Menu-Abmessungen: Nokia-Display
 *
 * Ein TFT mit den Abmessungen 84 x 48 Pixel wird gut ausgenutzt. Die Kopfzeile bietet Platz
 * fuer zwei Piktogramme 8x8 Pixel und eine Zeile Text. */
constflash Menudimension_t Menudimension_nokiaLcd =
{
    .Area = {0, 0, 84, 48},
    .headheight = 14,
    .scrollbarWidth = 5,
    .margin = 0,
    .textIndent = 1,
    .cornerRadius = 0,
    .headsep = 0,
    .textcellwidth = {30, 1, 20},
    .textalign = {align_left, align_center, align_right},
};



/** Menu-Farbschema: S/W-LCD
 *
 * Schwarze Kopfzeile, Piktrogramme weiss auf schwarzem Grund, Menuelement schwarz auf weiss */
constflash Menucolor_t ColorScheme_Monochrome =
{
    .mainBackground     = 0,
    .mainText = 1,
    .cursorText = 0,
    .cursorBackground = 1,
    .cursorFrame = transparent,
    .scrollbarActive = white,
    .scrollbarInactive = gray,
    .scrollbarFrame = red,
    .headlineBackground = 1,
    .headlineText = 0,
    .headlineIcon = 0,
};



/** Hilfsfunktion: Schleifen-Update-Funktion */
void menuLoopFcn(void)
{
    /* Da die Zeichen-Funktionen fuer GLCD mit Framebuffer den Framebuffer nicht automatisch an
       das LCD kopieren, wird dies in der Schleifenfunktion regelmaessig uebernommen. */
    glcd_drawnow();
}



/** Hilfsfunktion: Aktion ausloesen: Generische Aktions-Funktion in booleschen Wert umwandeln */
bool getAction(void)
{
    return !!ui_getAction();
}



/** Menue-Einstellungen: Standardmenue
 *
 *  * Zum Start wird der Hintergrund grau ausgeblendet
 *  * Der Drehgeber steuert Auswahl und Inkrement/Dekrement
 *  * Keine Schleifenfunktion */
constflash Menucontrol_t Menucontrol_standard =
{
    .pIncrFcn = ui_getIncrementFast,
    .pActnFcn = getAction,
    .pInitFcn = NULL,
    .pRptFcn  = menuLoopFcn,
};
