/** @file sw_config.h
 *
 * Software Config - Einstellungen , die keine Hardwareaenderungen benoetigen
 *
 * $Date: 2020-08-12 18:19:08 +0200 (Mi, 12 Aug 2020) $
 * $Revision: 67 $
 *
 * @date 30.03.2019 @author Nicolas
 */


#ifndef SW_CONFIG_H_INCLUDED
    #define SW_CONFIG_H_INCLUDED


    #if defined( TARGET_HARDWARE_STEPCNT_REV01 )
        #define DELAY_TARGET

    #elif defined( TARGET_UNIT_TEST_PC )
        #define INCLUDE_UNIT_TESTS
        #define DELAY_IGNORE
        #define INCLUDE_ADC_MOCKUP

    #elif defined( TARGET_MOCKUP_SDL )
        #define INCLUDE_UNIT_TESTS
        #define DELAY_SDL
        #define INCLUDE_MOCKUP_SDL
        #define INCLUDE_ADC_MOCKUP

    #endif



    /** @name Takte und Timer
     *
     * @{ */


    /** Prozessortakt (wird fuer PC-Emulation und Modultests nicht benoetigt) */
    #define SYSTEMCORECLOCK_HZ 72000000ULL

    /** Abtastrate/Aufruf-Rate Abtast-ISR  */
    #define SIGNAL_SAMPLE_RATE_HZ 500000ULL

    /** Zeitschrittweite SysTick-Timer.
        Beim SysTick-Timer wird im Gegensatz zu den anderen Zeitskalen das
        Intervall anstelle der Taktrate verwendet, weil gegen dieses oft gegengeprueft wird */
    #define SYSTICK_TIMER_INTERVAL_MS 1

    /** Timer-IRQ fuer den Highspeed-Timer (Abtast-ISR) */
    #define HIGHSPEED_TIMER_IRQn TIM4_IRQn

    /** @} */



    /** @name Buffers
     *
     * @{ */

    /** Hilfspuffer fuer Textstrings */
    #define TEXTBUFFERSIZE_BYTES 100

    /** Hilfspuffer fuer lokale Puffer mit MALLOC */
    #define AUX_BUFFERSIZE_BYTES 2048

    /** Pixelbuffer */
    #define PIXELBUFFERSIZE_BYTES 512

    /** @} */



    /** @name User Interface
     * @{ */

	/** Primitive Textfenster: Vordergrundfarbe */
    #define TEXT_COLOR black

    /** Primitive Textfenster: Hintergrundfarbe */
    #define BACKGROUND_COLOR green

    /** Schwellwert in Millisekunden, ab dem ein Tastendruck als "lang" erkannt wird */
    #define T_LONGKEYPRESS_MS 1500



    /** Menu-Inkrement herauf/herunter */
    #if defined( STM32F10x ) || defined( STM32F4XX )
        /* Auf den Zielplattformen bedeutet Rechtsdrehung Navigation nach unten */
        #define MENU_INKREMENT_DOWN
    #else
        /* Auf der PC-Emulation bedeutet Abwaerts-Taste Navigation nach unten */
        #define MENU_INKREMENT_UP
    #endif


    /* Schriftarten */
    /* Copyright Notice ../../3rd_party/src_3rd_party\src_u8g2_fonts/Fonts.txt */
    #define font_largedigit           u8g2_font_crox2cb_mn //u8g2_font_prospero_bold_nbp_tn
    #define font_monospaceTerminal    u8g2_font_courB08_tf
    #define font_largestFontAvailable font_largedigit
    #define font_breadAndButter       u8g2_font_baby_tf
    #define font_smallnice            u8g2_font_baby_tf
    #define font_menuDefault          font_breadAndButter
    #define font_test                 font_breadAndButter

    /** @} */


#endif
