/** @file Utest_delay.c
 *
 * Nutzer-interaktive Tests: Delays
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 25.03.2018 @author Nicolas */



#include "delay.h"
#include "hw_config.h"
#include "userinterface.h"
#include "io.h"



/** Wenn das Delay in Ordnung ist, ist auf dem Oszilloskop ein Rechtecksignal mit einer
 * Periodendauer von 20 Millisekunden zu sehen. */
void Utest_delay_ms(void)
{
    Ui_t Uim, *Ui = &Uim;
    ui_init(Ui, "Test delay_ms()");

    ui_clear();
    ui_printf("Test delay_ms() - no return");

    while( 1 )
    {
        delay_ms(10);
        io_setBit(SPARE1_PIN);
        delay_ms(10);
        io_clearBit(SPARE1_PIN);
    }

    ui_deinit(Ui);
}



/** Wenn das Delay in Ordnung ist, ist auf dem Oszilloskop ein Rechtecksignal mit einer
 * Periodendauer von 20 Millisekunden zu sehen. */
void Utest_delay_us(void)
{
    Ui_t Uim, *Ui = &Uim;
    ui_init(Ui, "Test delay_us()");

    ui_clear();
    ui_printf("Test delay_us() - no return");

    while( 1 )
    {
        delay_us(10000);
        io_setBit(SPARE1_PIN);
        delay_us(10000);
        io_clearBit(SPARE1_PIN);
    }

    ui_deinit(Ui);
}




#define TLOW_NS 10000000UL
#define THIGH_NS 10000000UL

/** Wenn das Delay in Ordnung ist, ist auf dem Oszilloskop ein Rechtecksignal mit einer
 * Periodendauer von 20 Millisekunden zu sehen. Einzelne Perioden sind laenger. */
void Utest_delay_ns(void)
{
    Ui_t Uim, *Ui = &Uim;
    ui_init(Ui, "Test delay_ns()");
    ui_clear();
    ui_printf("Test delay_ns() - no return");

    uint32_t l = TLOW_NS;
    uint32_t h = THIGH_NS;

    ui_printf("Test delay_ns()\nSpare 2\nLow=%u ns, High=%u ns\n", l, h);

    while( 1 )
    {
        io_clearBit(SPARE1_PIN);
        delay_ns(TLOW_NS);
        io_setBit(SPARE1_PIN);
        delay_ns(THIGH_NS);
        io_clearBit(SPARE1_PIN);
        delay_ns(TLOW_NS);
        io_setBit(SPARE1_PIN);
        delay_ns(THIGH_NS);
        io_clearBit(SPARE1_PIN);
        delay_ns(TLOW_NS);
        io_setBit(SPARE1_PIN);
        delay_ns(THIGH_NS);
        io_clearBit(SPARE1_PIN);
        delay_ns(TLOW_NS);
        io_setBit(SPARE1_PIN);
        delay_ns(THIGH_NS);
        io_clearBit(SPARE1_PIN);
        delay_ns(TLOW_NS);
        io_setBit(SPARE1_PIN);
        delay_ns(THIGH_NS);
    }

    ui_deinit(Ui);

}


