/** @file delay.c
 *
 * Delays: Hilfsdefinitionen fuer die Kompatibilitaet mit AVR-Code
 *
 * Es werden nur die langen Wartezeiten wirklich ausgefuehrt. Kurze Wartezeiten werden
 * ignoriert.
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 22.12.2013 @author Nicolas */


#include "delay.h"
#include "sw_config.h"


#if defined( DELAY_SDL )
    #include <SDL2/SDL.h>

    /** SDL-Emulation - Millisekunden verzoegern: Wird ignoriert
     * @param[in] a: ignoriert */
    void delay_ns(uint32_t a)
    {
        (void) (a);
    }



    /** SDL-Emulation - Mikrosekunden verzoegern: Wird ignoriert
     * @param[in] a: ignoriert */
    void delay_us(uint32_t a)
    {
        (void) (a);
    }



    /** SDL-Emulation - Millisekunden verzoegern
     * @param[in] milliseconds: Wartezeit in Millisekunden
     * Windows/Linux-Mockup - vorher noch einmal alle Inputs abarbeiten */
    void delay_ms(uint32_t milliseconds)
    {
        SDL_Delay(milliseconds);
    }



#elif defined( DELAY_IGNORE )

    /** Modultests - Nanosekunden verzoegern: Wird ignoriert
     * Bei den Modultests werden alle Wartezeiten ignoriert, um die Tests zu beschleunigen
     * @param[in] a: ignoriert */
    void delay_ns(uint32_t a)
    {
        (void) (a);
    }



    /** Modultests - Mikrosekunden verzoegern: Wird ignoriert
     * Bei den Modultests werden alle Wartezeiten ignoriert, um die Tests zu beschleunigen
     * @param[in] a: ignoriert */
    void delay_us(uint32_t a)
    {
        (void) (a);
    }



    /** Modultests - Millisekunden verzoegern: Wird ignoriert
     * Bei den Modultests werden alle Wartezeiten ignoriert, um die Tests zu beschleunigen
     * @param[in] a: ignoriert */
    void delay_ms(uint32_t a)
    {
        (void) (a);
    }


#else
    typedef int this_is_no_empty_translation_unit;

#endif



extern volatile uint32_t counter_milliseconds;


#if !PC_UNITTEST

    /** Zeit abwarten (Millisekunden)
     * Benoetigt laufenden Millisekunden-Zaehler
     *
     * delay_ms(): Zeit verzoegern, wait_ms(): Verstrichene Zeit abwarten.
     * Der Unterschied ist insbesondere bei hoher Interrupt-Last relevant.
     *
     * @param[in] milliseconds: Wartedauer */
    void wait_ms(uint32_t milliseconds)
    {
        uint32_t waitto = counter_milliseconds + milliseconds;
        while( (int32_t) (counter_milliseconds - waitto) < 0 );
    }

#else

    /** Modultests - Millisekunden abwarten: Wird ignoriert
     * Bei den Modultests werden alle Wartezeiten ignoriert, um die Tests zu beschleunigen
     * @param[in] milliseconds: ignoriert */
    void wait_ms(uint32_t milliseconds)
    {
        (void) milliseconds;
    }

#endif
