/** @file pgmspace.c
 *
 * Hilfsdefinitionen fuer die Kompatibilitaet mit AVR-Code: Progmem und Abhaengigkeiten
 *
 * und ein Workaround fuer eine fehlerhafte Implementierung von snprintf_P in ARV-GCC 4.8.1
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 22.12.2013 @author Nicolas */


#include "pgmspace.h"

#ifdef AVR


    #include <stddef.h>
    #include <stdio.h>
    #include <stdarg.h>

    #define GCC_VERSION ( __GNUC__ * 10000 \
            + __GNUC_MINOR__ * 100 \
            + __GNUC_PATCHLEVEL__ )

    #if ( (GCC_VERSION>=40800 && GCC_VERSION<40804) || GCC_VERSION==40900)

        #warning "This AVR-GCC version needs workaround for flash strings"

        /** Workaround-Funktion fuer snprintf_P
         *
         * WORKAROUND:
         *  Bug 61443
         *  https://gcc.gnu.org/bugzilla/show_bug.cgi?id=61443
         *
         *  Unter AVR-GCC 4.8.1 sorgt die direkte Verwendung von
         *  "menudata[item].menutext" mit "snprintf_P" fuer einen Compilerfehler
         *  ("unrecognizable insn"). Konkret lautete die Zeile:
         *
         *  snprintf_P(gl_stringbuffer,nStringbuffer,menudata[item].menutext,*menudata[item].puint8);
         *
         *  Als Workaround wird der String zuerst ins SRAM kopiert und anschliessend
         *  die snprintf-Funktion genutzt (benoetigt 81 Bytes zusaetzlich).
         *
         * WORKAROUND: Format-String erst in SRAM-Puffer kopieren und dann mit normalem snprintft
         * bearbeiten
         *
         * @param[out] dst: Zielstring
         * @param[in] size: Groesse Zielstring
         * @param[in] Format-String
         * @return Number of characters printed */
        int wasnprintf_P(char *dst, size_t size, constflash char * format, ... )
        {
            va_list args;

            int ret;

            char tempstr[size+1];
            strncpy_P(tempstr, format, nStringbuffer-1);
            tempstr[size] = '\0';

            va_start(args, format);
            ret = vsnprintf(dst, size, tempstr, format, args);
            va_end(args);

            return ret;
        }
    #else

        /** Normaler Ersatz, der snprintf_P direkt aufruft @see snprintf */
        int wasnprintf_P(char *dst, size_t size, constflash char * format, ...)
        {
            int ret;
            va_list args;
            va_start(args, format);
            ret = vsnprintf_P(dst, size, format, args);
            va_end(args);
            return ret;
        }

    #endif







#else
    typedef int this_is_no_empty_translation_unit;
#endif


