/** @file pgmspace.h
 *
 * Hilfsdefinitionen fuer die Kompatibilitaet mit AVR-Code: Progmem und Abhaengigkeiten
 *
 * und ein Workaround fuer eine fehlerhafte Implementierung von snprintf_P in ARV-GCC 4.8.1
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 22.12.2013 @author Nicolas */



#ifndef PGMSPACE_H
    #define PGMSPACE_H


    #ifdef AVR
	    #define constflash const __flash
		#include <avr/pgmspace.h>
        #include <stdio.h>

	#elif defined(STM32F10x) || defined(STM32F4XX) || defined __MINGW64__ \
        || defined __MING32__ || defined __linux__
        #define PGMSPACE_EMULATION 1

    #else
        #error "undefined platform"

    #endif


    #if PGMSPACE_EMULATION

		#define PROGMEM
		#define pgm_read_byte(a) (*(a))
		#define pgm_read_word(a) (*(a))
		#define pgm_read_pointer(a) (*(a))
        #define constflash const
		#define PSTR(a) (a)

		#define strlen_P strlen
		#define strncpy_P strncpy

		#define vsnprintf_P vsnprintf
		#define printf_P printf
        #define fprintf_P fprintf
		#define fputs_P fputs


        /** Fehler erzeugen, wenn Funktion snprintf_P() aufgerufen wird.
         *
         * snprintf_P sollte wegen eines Fehlers in einer AVR-GCC-Version nicht verwendet werden.
         * Als Workaround wird die Funktion wasnprintf_P eingefuehrt. */
		#define snprintf_P 1

		/** Workaround-snprintf-Funktion */
		#define wasnprintf_P snprintf

	#endif

#endif


