/** @file encoder_mocksdl.c
 *
 * SDL-Mockup: Auslesen eines Drehgebers
 *
 * Die Funktion encoder_get() pollt den Drehgeber ohne !negative! Nebenwirkungen und kann
 * beliebig oft aufgerufen werden.
 * encoder_get_diff() gibt Werte zwischen aufeinanderfolgenden Aufrufen zurueck.
 * Mit encoder_get() in einer schnellen Programmschleife oder ISR und encoder_get_diff()
 * in einer langsamen Programmschleife laesst sich eine schnelle Benutzerschnittstelle
 * realisieren.
 *
 *
 * $Date: 2020-08-12 16:39:52 +0200 (Mi, 12 Aug 2020) $
 * $Revision: 66 $
 *
 * @date 19.12.2013 @author Nicolas */


#include "encoder_mocksdl.h"
#include "sw_config.h"

#if defined( INCLUDE_MOCKUP_SDL )

    #include <stdbool.h>
    #include <stdint.h>
    #include "mocksdl_input.h"
    #include "mocksdl.h"
    #include "myassert.h"


    /** Die Variable glSdlenc wird von den SDL-Poll-Funktionen zur Verfugung gestellt. Hier wird
     * sie nur ausgelesen. */
    extern uint_fast8_t glSdlenc;



    /** Differenz des Drehgeber-Standes zwischen aufeinanderfolgenden Aufrufen der Funktion
     + auslesen. NICHT re-entrant!
     *
     * @return: Differenz zu vorherigem Aufruf */
    int_fast8_t mocksdl_encoder_get_diff(void)
    {
        static uint_fast8_t enc_last;
        uint_fast8_t enc_this;
        int_fast8_t enc_diff;

        static_assert(sizeof(uint_fast8_t)==sizeof(int_fast8_t), \
                "variable must be of same size");

        enc_this = mocksdl_encoder_get();
        enc_diff = (int_fast8_t) (enc_this-enc_last);
        enc_last = enc_this;

        return enc_diff;
    }



    /** Drehgeber: Initialisierung (Dummy), hier: Wert ruecksetzen */
    void mocksdl_encoder_init(void)
    {
        glSdlenc = 0;
    }



    /** Drehgeber pollen und/oder auslesen
     *
     * Drehgeber pollen, Absolutwert abfragen und Statusupdate vollziehen.
     *
     * @return: Aktueller Wert  */
    uint_fast8_t mocksdl_encoder_get(void)
    {
        /* Encoder-Wert updaten
         * sowohl pollen fuer internen Statusupdate als auch Auslesen der internen
         * Werte */
        mock_event_poll();
        return glSdlenc;
    }


#else
    typedef int this_is_no_empty_translation_unit;
#endif


