/** @file color.c
 *
 * RBG-Farben, insbesondere 16 Bit
 *
 * $Date: 2020-08-02 19:08:25 +0200 (So, 02 Aug 2020) $
 * $Revision: 32 $
 *
 * @date 30.09.2017 @author Nicolas */


#include "color.h"
#include "myassert.h"






/** Bei jedem Aufruf eine andere Farbe ausgeben */
color_t color_rainbow(void)
{
    static uint8_t i = 0;

    switch( (i++)%14 )
    {
        case 0: return blue;
        case 1: return red;
        case 2: return green;
        case 3: return cyan;
        case 4: return magenta;
        case 5: return yellow;
        case 6: return white;
        case 7: return black;
        case 8: return gray;
        case 10: return violet;
        case 11: return silver;
        case 12: return gold;
        case 13: return brown;
        case 14: return orange;
    }

    return 0;
}



/** Zwei Farben mischen/inpterpolieren
 *
 * @param[in] Colors: Farbenpaar (16 Bit) Vordergrund-/Hintergrundfarbe
 * @param[in] ratio: [0...256] 256 = 100% Vordergrundfarbe, 0 = 100% Hintergrundfarbe
 * @return gemischter Wert */
color_t color_interpolate(Colors_t Colors, uint_fast16_t ratio )
{
    const uint32_t base = 256;
    const uint32_t rmask = 0xF800;
    const uint32_t gmask = 0x07E0;
    const uint32_t bmask = 0x001F;

    assert( ratio <= base );

    color_t c2 = Colors.bg;
    color_t c1 = Colors.fg;

    uint32_t r = (uint32_t) (c1 & rmask)*ratio + (uint32_t) (c2 & rmask) * (base-ratio);
    uint32_t g = (uint32_t) (c1 & gmask)*ratio + (uint32_t) (c2 & gmask) * (base-ratio);
    uint32_t b = (uint32_t) (c1 & bmask)*ratio + (uint32_t) (c2 & bmask) * (base-ratio);

    color_t result = 0;
    result |= (r/base) & rmask;
    result |= (g/base) & gmask;
    result |= (b/base) & bmask;
    return result;
}

