/** @file color.h
 *
 * RGB-Farben, insbesondere 16 Bit
 *
 * $Date: 2020-08-02 19:08:25 +0200 (So, 02 Aug 2020) $
 * $Revision: 32 $
 *
 * @date 30.09.2017 @author Nicolas */



#ifndef COLOR_H
    #define COLOR_H

    #include "nstdint.h"


    /** 16-Bit RGB (5 Bit R, 6 Bit G, 5 Bit B) */
    enum color16_e
    {
        black       = 0x0000,
        blue        = 0x001F,
        red         = 0xF800,
        green       = 0x07E0,
        cyan        = 0x07FF,
        magenta     = 0xF81F,
        yellow      = 0xFFE0,
        white       = 0xFFFF,
        gray        = 0x8410,
        violet      = 0xEC1D,
        silver      = 0xC618,
        gold        = 0xFEA0,
        brown       = 0xA145,
        orange      = 0xFD20,
        transparent = 0x0002, /**< magic value. Nicht 0001, weil 1 bei S/W-GLCDs andere Bedeutung hat  */
    };



    typedef uint16_t color_t;


    /** Farbenpaar Vordergrund/Hintergrund */
    typedef struct Colors_s
    {
        uint16_t fg; /// Vordergrundfarbe
        uint16_t bg; /// Hintergrundfarbe
    }
    Colors_t;


    /** Aufgeteilte Kanaele R,G,B */
    typedef struct Color_Rgb24_s
    {
        uint8_t r, g, b;
    }
    Color_Rgb24_t;



    color_t color_rainbow(void);
    color_t color_interpolate(Colors_t Color, uint_fast16_t x );




    /** 16-Bit-RGB in Komponenten (24 Bit) umwandeln
     *
     * @param[in] rgb: 16-Bit-RGB (5 Bit R, 6 Bit G, 5 Bit B)
     * @return separate Kanaele r, g, b je 8 Bit */
    static inline Color_Rgb24_t color_rgb16to24_s(uint16_t rgb)
    {
        Color_Rgb24_t Rgb;
        Rgb.r = (rgb & 0xF800) >> 8;
        Rgb.g = (rgb & 0x07E0) >> 3;
        Rgb.b = (rgb & 0x001F) << 3;
        return Rgb;
    }


    /** 16-Bit-RGB in Komponenten (24 Bit) umwandeln (int) */
    static inline uint_fast24_t color_rgb16to24(uint16_t rgb16)
    {
        uint_fast24_t rgb24 = 0;
        rgb24 |= (rgb16 & 0xF800) << 8;
        rgb24 |= (rgb16 & 0x07E0) << 5;
        rgb24 |= (rgb16 & 0x001F) << 3;
        return rgb24;
    }


    /** 24-Bit-RGB-Komponenten in 16-Bit umwandeln, LSBs werden einfach abgeschnitten */
    static inline uint16_t color_rgb24to16(Color_Rgb24_t Rgb)
    {
        uint16_t rgb = 0;
        rgb |= (Rgb.r & 0xF8) << 8;
        rgb |= (Rgb.g & 0xFC) << 3;
        rgb |= (Rgb.b & 0xF8) >> 3;
        return rgb;
    }


#endif
