/** @file coord.h
 *
 * 2-D-Koordinaten (Integer-Operationen)
 *
 * $Date: 2020-08-01 09:01:46 +0200 (Sa, 01 Aug 2020) $
 * $Revision: 27 $
 *
 * @date 1.11.2017 @author Nicolas */

#ifndef COORD_H
    #define COORD_H

    #include <stdint.h>
    #include <stdbool.h>

    /** 2-D-Koordinaten
     *
     * Definition: Dieser Datentyp ist immer gross genug, die maximalen und minimalen
     * x- und y-Koordinaten des jeweils verwendeten Display-Typs zu halten. */
    typedef struct Coord2_s
    {
        int16_t x, y;
    }
    Coord2_t;

    #define COORD2_MAX INT16_MAX
    #define COORD2_MIN INT16_MIN


    /** Addition zweier Vektoren
     *
     * @param[in] a, b: Summanden
     * @return Summe */
    static inline Coord2_t coord2_add(Coord2_t a, Coord2_t b)
    {
        return (Coord2_t) {a.x + b.x, a.y + b.y};
    }


    /** Subtraktion zweier Vektoren
     *
     * @param[in] a: Minuend
     * @param[in] b: Subtrahend
     * @return Differenz */
    static inline Coord2_t coord2_sub(Coord2_t a, Coord2_t b)
    {
        return (Coord2_t) {a.x - b.x, a.y - b.y};
    }


    /** Gleichheit zweier Vektoren
     *
     * @param[in] a, b: Vektoren
     * @return Gleichheit */
    static inline bool coord2_eq(Coord2_t a, Coord2_t b)
    {
        return (a.x == b.x) && (a.y == b.y);
    }



    uint_fast8_t coord2_sort(Coord2_t *Pos, int nEl, char dir);




#endif /* COORD_H */
