/** @file gfx_draw_diagrams.c
 *
 * Grafikfunktionen fuer GLCD und TFT: Einfache zweifarbige Diagramme
 *
 * Die Funktionen eigenen sich fuer Farb- und Monochrom-LCD gleichermassen, da der Farbvektor
 * einfach an die Low-Level-Grafikfunktion durchgereicht wird.
 *
 * $Date: 2020-08-02 19:08:25 +0200 (So, 02 Aug 2020) $
 * $Revision: 32 $
 *
 * @date 29.03.2018 @author Nicolas */


#include "gfx_draw_diagrams.h"
#include "gfx_draw_lines.h"
#include "gfx_draw_rectangles.h"



/** Bounding-Box als Fensterkreuz darstellen.
 *
 * @param[in] Pos: Position der oberen linken Ecke
 * @param[in] BB: Bounding-Box im ueblichen Format mit x, y als intenen Offsets */
void drawBoundingBox(Coord2_t Pos, Rect_t BB, Colors_t Color)
{
    Rect_t R = { .x = Pos.x, .y = Pos.y, .w = BB.w, .h = BB.h, };
    drawDottedRect(R, (Colors_t) {Color.fg, transparent});
    Coord2_t H0, H1, V0, V1;
    H0.y = Pos.y - BB.y;
    H1.y = H0.y;
    H0.x = Pos.x;
    H1.x = Pos.x + BB.w;
    V0.x = Pos.x - BB.x;
    V1.x = V0.x;
    V0.y = Pos.y;
    V1.y = Pos.y + BB.h;

    drawDottedLine(H0, H1, Color.bg);
    drawDottedLine(V0, V1, Color.bg);
}
