/** @file gfx_draw_triangles.c
 *
 * Grafikfunktionen fuer GLCD und TFT: Dreiecke
 *
 * Die Funktionen eigenen sich fuer Farb- und Monochrom-LCD gleichermassen, da der Farbvektor
 * einfach an die Low-Level-Grafikfunktion durchgereicht wird.
 *
 * $Date: 2020-08-05 10:30:04 +0200 (Mi, 05 Aug 2020) $
 * $Revision: 43 $
 *
 * @date 1.11.2017 @author Nicolas */


#include "gfx_draw_triangles.h"
#include "gfx_draw_lines.h"
#include "gfx_draw.h"
#include "intmath.h"
#include "nstdint.h"



/** Dreieck (Umriss) zeichnen
 *
 * @param[in] Pos0, Pos1, Pos2: Ecken (x/y-Koordinaten)
 * @param[in] color: 16-Bit-Farbe */
void drawTriangle(Coord2_t Pos0, Coord2_t Pos1, Coord2_t Pos2, uint16_t color )
{
    drawLine(Pos0, Pos1, color );
    drawLine(Pos1, Pos2, color );
    drawLine(Pos2, Pos0, color );
}




/** Gefuelltes Dreieck zeichnen
 *
 * Fuer jede Zeile finden zwei Divisionen statt
 * @param[in] Pos0, Pos1, Pos2: Ecken (x/y-Koordinaten)
 * @param[in] Color: 16-Bit-Farbe fg: Rand, bg: Fuellung
 *
 * @todo (wenn Langeweile): Gefuellte Dreiecke werden ueber den Rand gemalt */
void drawTriangleF(Coord2_t Pos0, Coord2_t Pos1, Coord2_t Pos2, Colors_t Color)
{
    uint_fast16_t color = Color.fg;

    /* Punke nach x sortieren (Bubble-Sort) */
    Coord2_t Pos[3] = {Pos0, Pos1, Pos2 };
    coord2_sort(Pos, 3, 'x');

    if( Color.bg != transparent )
    {
        drawTriangle(Pos[0], Pos[1], Pos[2], Color.bg);
    }



    /* Kante A */
    Coord2_t A = Pos[0], dA = coord2_sub(Pos[2], Pos[0]); /* x, y, dx, dy */
    Coord2_t B, dB;
    int rA = 0, rB; // Rest


    drawRectF(coordToRect(A, (Coord2_t) {1, MAX(1,1) }), color ); /* Erstes Pixel */

    for( int j = 0; j < 2; j++)
    {

        /* Kante B (unterbrochen) */
        if( j == 0 )
        {
            B = Pos[0];
            dB = coord2_sub(Pos[1], Pos[0]);
            rB = 0;
        }
        else
        {
            B = Pos[1];
            dB = coord2_sub(Pos[2], Pos[1]);
            rB = 0;
        }

        /* Zeilenweise durchschreiten */
        for( int i = 0; i < dB.x; i++ )
        {
            if( dA.x != 0 )
            {
                A.y += (dA.y+rA)/dA.x;
                rA = (dA.y+rA)%dA.x;
            }
            A.x++;

            /* dB.x != 0 (siehe Schleifenbedingung) */
            B.y += (dB.y+rB)/dB.x;
            rB = (dB.y+rB)%dB.x;
            B.x++;

            drawRectF(coordToRect(A, (Coord2_t) {1, B.y-A.y +1}), color );
        }
    }

    // Nur zum testen
    //drawTriangle(Pos[0], Pos[1], Pos[2], yellow);
}
