/** @file Utest_gfx_color_testpattern.c
 *
* Nutzer-interaktive Tests: Testbilder zu Grafikfunktionen
 *
 * $Date: 2020-08-04 18:20:49 +0200 (Di, 04 Aug 2020) $
 * $Revision: 41 $
 *
 * @date 04.03.2018 @author Nicolas */


#include "Utest_gfx_color.h"
#include "gfx_draw_rectangles.h"
#include "gfx_draw_triangles.h"
#include "gfx_draw_circles.h"
#include "gfx_draw_lines.h"
#include "gfx_draw.h"
#include "userinterface.h"



/** Testbild 1: Linienlaengen
 * Rote Linien gehen von oben nach unten, weisse von unten nach oben
 * Beide sollen zusammenpassen
 * Das Dreieck muss spitz sein (Linie der Laenge 1 auch dargestellt) */
void Utest_vertical_lines(void)
{
    Coord2_t Start = {30, 10};
    int nTest = 100;

    Coord2_t Lr0, Lr1, Lw0, Lw1;
    Lr0 = Start;
    Lr1 = Start;
    Lw0.x = Start.x + 1, Lw0.y = Start.y+1;
    Lw1.x = Start.x + 1, Lw1.y = Start.y;

    for( int i = 0; i<nTest; i++ )
    {
        drawLine(Lr0, Lr1, red );
        drawLine(Lw0, Lw1, white );

        Lr0.x +=2; Lr1.x +=2; Lw0.x +=2; Lw1.x +=2;
        Lr1.y +=2;
        Lw0.y +=2;
    }
}



/** Testbild 2: Linienlaengen
 * Gruene Linien gehen von links nach rechts, gelbe von rechts nach links
 * Beide sollen zusammenpassen
 * Das Dreieck muss spitz sein (Linie der Laenge 1 auch dargestellt) */
void Utest_horizontal_lines(void)
{
    Coord2_t Start = {10, 30};
    int nTest = 100;

    Coord2_t Lg0, Lg1, Ly0, Ly1;
    Lg0 = Start;
    Lg1 = Start;
    Ly0.x = Start.x + 1, Ly0.y = Start.y+1;
    Ly1.x = Start.x,     Ly1.y = Start.y+1;

    for( int i = 0; i<nTest; i++ )
    {
        drawLine(Lg0, Lg1, green );
        drawLine(Ly0, Ly1, yellow );

        Lg0.y +=2; Lg1.y +=2; Ly0.y +=2; Ly1.y +=2;
        Lg1.x +=2;
        Ly0.x +=2;
    }
}



/** Testbild 3: Rechteck mit abgerundeten Ecken */
void Utest_drawRectRounded(void)
{
    Rect_t R = {.x = 250, .y=40, .w = 60, .h = 160};
    int radius = 100;
    Colors_t Color = {red, 0xE571};

    drawRectRounded(R, radius, Color);
}



/** Testbilder 1, 2 und 3 in gemeinsamem Screen
 * Beenden durch Tastendruck */
void Utestgroup_linesAndRect(void)
{
    Utest_vertical_lines();
    Utest_horizontal_lines();
    Utest_drawRectRounded();

    while( !ui_getAction() );
}



/** Testbildausschnitt 4: Farbbalken, Linien, Kreise
 *
 * @param[in] Rtot: Flaeche
 * @param[in] dir: 0|1 senkrechter/waagerechter Farbverlauf
 * @return EXIT_SUCCESS */
exit_t drawTestpatternGradient(Rect_t Rtot, uint_fast8_t dir)
{
    const uint_fast8_t nBar = 7; /* inkl. Graukeil */
    Rect_t Rbar = {0, 0, 0, 0};
    int_fast16_t dx, dy, x0, y0;

    if( dir == 0 )
    {
        Rbar.w = Rtot.w/nBar;
        Rbar.h = Rtot.h;
        dx = Rbar.w;
        dy = 0;
        x0 = Rtot.w - dx*nBar; /* Rand (l+r) */
        y0 = 0;
    }
    else
    {
        Rbar.w = Rtot.w;
        Rbar.h = Rtot.h/nBar;
        dx = 0;
        dy = Rbar.h;
        x0 = 0;                /* Rand (l+r) */
        y0 = Rtot.h - dy*nBar; /* Rand (o+u) */
    }



    /* Graukeil */
    Rbar.x += x0/2;
    Rbar.y += y0/2;

    drawRectGradient(Rbar, (Colors_t) {black, white}, dir);

    uint16_t basecolor;

    if( dir == 0 )
        Rbar.h /= 2;
    else
        Rbar.w /= 2;

    for( uint8_t i = 1; i<nBar; i++  )
    { /* 1. Spalte wurde ja schon gezeichnet */
        switch( i )
        {
            case 1:
                basecolor = red;
                break;
            case 2:
                basecolor = green;
                break;
            case 3:
                basecolor = blue;
                break;
            case 4:
                basecolor = cyan;
                break;
            case 5:
                basecolor = magenta;
                break;
            case 6:
                basecolor = yellow;
                break;
        }

        Rbar.x  += dx;
        Rbar.y  += dy;
        drawRectGradient(Rbar, (Colors_t) {basecolor, white}, dir);
        if( dir == 0 )
            Rbar.y  += Rtot.h - Rbar.h;
        else
            Rbar.x  += Rtot.w - Rbar.w;
        drawRectGradient(Rbar, (Colors_t) {black, basecolor}, dir);
        if( dir == 0 )
            Rbar.y = 0;
        else
            Rbar.x = 0;
    }

    return EXIT_SUCCESS;
}



/** Testbildausschnitt: Linienmuster
 *
 * @param[in] Rtot: Flaeche
 * @return EXIT_SUCCESS */
exit_t drawTestpatternLines(Rect_t Rtot)
{

    void (*drawFcn) (Coord2_t, Coord2_t, color_t) = drawDottedLine;


    int16_t shortlen = MIN(Rtot.w, Rtot.h);


    /* Kreis */
    Coord2_t C = {Rtot.x + Rtot.w/2, Rtot.y + Rtot.h/2};
    int r = shortlen/8*3;
    drawCircle(C, r, white);
    drawCircle(C, r-1, white);

    /* Diagonalen */
    Coord2_t P0, P1;
    P0.x = Rtot.x;
    P0.y = Rtot.y;
    P1 = coord2_add(P0, (Coord2_t) {shortlen, shortlen} );
    drawFcn(P0, P1, white);
    P0.x = Rtot.x + Rtot.w;
    P0.y = Rtot.y;
    P1 = coord2_add(P0, (Coord2_t) {-shortlen, shortlen} );
    drawFcn(P0, P1, white);
    if( Rtot.w != Rtot.h )
    {
        P0.x = Rtot.x;
        P0.y = Rtot.y  + Rtot.h;
        P1 = coord2_add(P0, (Coord2_t) {shortlen, -shortlen} );
        drawFcn(P0, P1, white);
        P0.x = Rtot.x + Rtot.w;
        P0.y = Rtot.y + Rtot.h;
        P1 = coord2_add(P0, (Coord2_t) {-shortlen, -shortlen} );
        drawFcn(P0, P1, white);
    }

    /* Horizontale und vertikale Linien */
    {
        Coord2_t P0, P1, P2, P3;
        ifast16_t rn = r/5*4;
        P0.x = C.x - rn;
        P0.y = C.y - rn;
        P1.x = C.x - rn;
        P1.y = C.y + rn;
        P2.x = C.x + rn;
        P2.y = C.y + rn;
        P3.x = C.x + rn;
        P3.y = C.y - rn;
        drawFcn(P0, P1, white);
        drawFcn(P1, P2, white);
        drawFcn(P2, P3, white);
        drawFcn(P3, P0, white);
    }

    return EXIT_SUCCESS;
}



/** Testbilder: Farbverlaufsbalken und Linienmuster uebereinander */
void Utestgroup_gradient(void)
{
    drawTestpatternGradient(gfx_getDisplayDimensions(), 0);
    drawTestpatternLines(gfx_getDisplayDimensions());

    while( !ui_getAction() );
}



/** Testbildausschnitt: Noise-Hintergrund und Dreiecke
 *
 * @param[in] Rtot: Flaeche
 * @return EXIT_SUCCESS */
uint_fast8_t drawTestpatternTriangles(Rect_t Rtot)
{
    (void) Rtot; /* Erst einmal Rechteck nicht einstellbar */

    /* "normales Dreieck */
    drawTriangleF((Coord2_t) {30, 10 }, (Coord2_t) {200, 30 }, (Coord2_t) {100, 150 }, (Colors_t) {cyan, transparent} );

    /* Sonderfall: Horizontale Kante, dadurch intern nur eine Flaeche */
    drawTriangleF((Coord2_t) {30, 30 }, (Coord2_t) {200, 30 }, (Coord2_t) {200, 150 }, (Colors_t) {red, transparent} );

    /* Sonderfall: kein Dreieck */
    drawTriangleF((Coord2_t) {30, 30 }, (Coord2_t) {30, 30 }, (Coord2_t) {200, 150 }, (Colors_t) {blue, transparent} );

     /* Sonderfall: Horizontale Kante, dadurch intern nur eine Flaeche (nach unten) */
    drawTriangleF((Coord2_t) {30, 30 }, (Coord2_t) {30, 200 }, (Coord2_t) {200, 150 }, (Colors_t) {magenta, yellow} );

    /* Leeres Dreieck */
    drawTriangle((Coord2_t) {-10, 10 }, (Coord2_t) {200, 30 }, (Coord2_t) {200, 10 }, yellow );

    return EXIT_SUCCESS;
}



/** Test: Bildschirminhalt kopieren
 *
 * (nicht implementiert fuer Monochrom-LCD) */
void Utest_copyScreen(void)
{
    #if defined( INCLUDE_TFT_FUNCTIONS )

        Rect_t Src = {4, 140, 150, 150};
        Rect_t Dst = {120, 50, 150, 150};

        drawRect(Src, (Colors_t) {yellow, transparent} );
        tft_copyregion(Src, Dst);

        drawRect(Dst, (Colors_t) {green, transparent} );
    #endif
}



/** Testbild: Rauschen im Hintergrund, Dreicke, Teil des Bildschirminhalts kopieren */
void Utestgroup_noiseAndCopy(void)
{
    drawRectNoise(gfx_getDisplayDimensions());

    drawTestpatternTriangles(gfx_getDisplayDimensions());

    Utest_copyScreen();
    while( !ui_getAction() );

}
