/** @file glcd_generic.h
 *
 * Generische Funkionsnamen fuer SW-Grafik-LCDs mit maximal 255 x  255 Pixel
 *
 * Alle Grafik-Displays werden (nur noch) mit Framebuffer unterstuetzt.
 *
 * Created: 15.02.2016
 *  Author: Nicolas
 */


#ifndef GLCD_GENERIC_H
    #define GLCD_GENERIC_H

    #include <stdint.h>
    #include "coord.h"
    #include "hw_config.h"


    #ifndef static_inline
        #define static_inline static inline __attribute__((always_inline))
    #endif



    #if defined( GLCD_NONE )
        /* Kein Grafik-LCD vorhanden, reine Dummy-Funktionen */
        #define GLCD_END 0
        #define GLCD_WIDTH_PX 0
        #define GLCD_HEIGHT_PX 0


        /** Kein Display initialisieren */
        static_inline void glcd_init(void)
        {
            ;
        }


        /** Framebuffer nirgendwohin kopieren */
        static_inline void glcd_drawnow(void)
        {
            ;
        }


	#elif defined( GLCD_PCD8544 )
        #define GLCD_END      GLCD_PCD8544_END
        #define GLCD_WIDTH_PX GLCD_PCD8544_WIDTH_PX
        #define GLCD_HEIGHT_PX 48

        #include "glcd_pcd8544.h"


        /** Physisches Display initialisieren */
        static_inline void glcd_init(void)
        {
            glcd_pcd8544_init();
        }



        /** Framebuffer in den Display-Speicher kopieren */
        static_inline void glcd_drawnow(void)
        {
            glcd_pcd8544_drawnow();
        }


    #elif defined( GLCD_EMULATION_84x48 )
        #define GLCD_END (84*48/8)
        #define GLCD_WIDTH_PX 84
        #define GLCD_HEIGHT_PX 48
        #define GLCD_EMULATION_SDL

        #include "mocksdl_display.h"
        #include "glcd_mocksdl.h"

        /** Emuliertes Display initialisieren */
        static_inline void glcd_init(void)
        {
            mocksdl_display_init();
        }



        /** Framebuffer in den Display-Speicher kopieren */
        static_inline void glcd_drawnow(void)
        {
            glcd_mocksdl_drawFramebuffer();
        }


    #else
        void glcd_drawnow(void);
        uint16_t glcd_getpos(void);
        void glcd_setpos(uint16_t Pos);

    #endif

    /* glcd_framebuffer.h wird erst hier eingefuegt, damit GLCD_END, das von diesem Header benoetigt
       wird, schon definiert ist */
    #include "glcd_framebuffer.h"


    static_inline void glcd_putdat(uint8_t value)
    {
        glcd_frmbuf_putdat(value);
    }



    static_inline void glcd_clear(void)
    {
        glcd_frmbuf_clear();
    }



    static_inline void glcd_setpos(uint16_t pos)
    {
        glcd_frmbuf_setpos(pos);
    }



    static_inline uint16_t glcd_getpos(void)
    {
        return glcd_frmbuf_getpos();
    }


#endif
