/** @file glcd_mocksdl.c
 *
 * GLCD-Emulation per SDL: Display-Update
 *
 * $Date: 2020-08-12 18:19:08 +0200 (Mi, 12 Aug 2020) $
 * $Revision: 67 $
 *
 * @date 11.06.2020 @author Nicolas */



#include "sw_config.h"
#include "glcd_generic.h"
#include "glcd_mocksdl.h"


#if defined( GLCD_EMULATION_SDL )

    #include <SDL2/SDL.h>


    extern SDL_Renderer * pRenderer;
    extern SDL_Texture * pTexture;



    #include <stdio.h>
    #include <string.h>
    #include "glcd_generic.h"
    #include "rect.h"



    /** Framebuffer auf dem virtuellen Display darstellen
     * Der Framebuffer ist spaltenweise 8 Bit organisiert */
    void glcd_mocksdl_drawFramebuffer(void)
    {
        Rect_t R = {.x = 0, .y= 0, .w = GLCD_WIDTH_PX, .h = GLCD_HEIGHT_PX};


        /* Pixel-Puffer (16 Bit) bereitstellen */
        size_t nPixel = (size_t) R.w * R.h;
        uint16_t pixelbuffer[nPixel];
        memset(pixelbuffer, 0, sizeof(pixelbuffer[0])*nPixel);

        int pitch = R.w;
        uint8_t *framebuffer = gl_framebuffer;


        /* Matrix-Display-Anordnung in SDL-Matrix umwandeln */
        for( size_t i = 0; i < GLCD_END; i++)
        {
            /* Spaltenblock mit 8 Pixeln (B/W) */
            uint8_t blk = *framebuffer++;

            int col = i%GLCD_WIDTH_PX;
            int row = i/GLCD_WIDTH_PX;

            for( uint_fast8_t bit = 0; bit < 8; bit++ )
            {
                size_t idx = (size_t) GLCD_WIDTH_PX*8UL*row + col + (size_t) GLCD_WIDTH_PX*bit;
                assert( idx < nPixel );

                if( blk & 0x01 )
                {
                    pixelbuffer[idx]=0; /* schwarz */
                }
                else
                {
                    pixelbuffer[idx]=0xFFFF; /* weiss */
                }
                blk>>=1;
            }
        }


        SDL_Rect Rect = {.x = R.x, .y = R.y, .w = R.w, .h = R.h};
        pitch *= sizeof(uint16_t);

        SDL_UpdateTexture(pTexture, &Rect, pixelbuffer, pitch);
        SDL_RenderCopy(pRenderer, pTexture, &Rect, &Rect);
        SDL_RenderPresent(pRenderer);
    }


#else
    typedef int this_is_no_empty_translation_unit;
#endif
