/** @file io.h
 *
 * GPIO-Operationen
 * (+ Hilfsdefinitionen fuer die Kompatibilitaet zwischen STM32- AVR- und Mockup-Code)
 *
 * $Date: 2020-08-01 09:01:46 +0200 (Sa, 01 Aug 2020) $
 * $Revision: 27 $
 *
 * @date 22.02.2014 @author Nicolas */


#ifndef AVRCOM_IO_H
    #define AVRCOM_IO_H

    #include "hw_config.h"

    #if !( defined( GPIO_HARDWARE ) || defined( GPIO_EMULATION ) )
        #error "GPIO must bei either real hardware or emulated"
    #endif


    #ifdef AVR
	    #include <avr/io.h>
		#include "io_avr.h"

        /** GPIO-Register */
        typedef	volatile uint8_t GPIO_t;

        /** GPIO-Pins (Bitmaske) */
        typedef	uint8_t IOPin_t;


    #elif defined(STM32F10x)
        #include "io_stm32f10x.h"
        #include "io_pins.h"

    #elif defined(STM32F4XX)
        #include "io_stm32f4xx.h"
        #include "io_pins.h"

	#elif defined( GPIO_EMULATION )
		#include "io_mock.h"
        #include "io_pins.h"

    #else
        #error "unknown Platform"

    #endif


#endif // AVRCOM_IO_H
