/** @file io_mock.c
 *
 * GPIO-Operationen-Mockup
 *
 * $Date: 2020-08-01 09:01:46 +0200 (Sa, 01 Aug 2020) $
 * $Revision: 27 $
 *
 * @date 22.02.2014 @author Nicolas  */

#include "io_mock.h"
#include "hw_config.h"

#if defined( GPIO_EMULATION ) || defined( DOXYGEN )

    #include <stdio.h>

    /** Zustaende aller emulierten GPIO-Ports */
    GPIO_t GpioValue[5];


    GPIO_t *GPIOA = &(GpioValue[0]);    /**< GPIOA (Mockup) */
    GPIO_t *GPIOB = &(GpioValue[1]);    /**< GPIOB (Mockup) */
    GPIO_t *GPIOC = &(GpioValue[2]);    /**< GPIOC (Mockup) */
    GPIO_t *GPIOD = &(GpioValue[3]);    /**< GPIOD (Mockup) */
    GPIO_t *GPIOE = &(GpioValue[4]);    /**< GPIOE (Mockup) */




    /** Takt/Clock fuer GPIO bereitstellen (Dummy-Funktion)
     *
     * Die Information ueber den Aufruf wird in der Konsole ausgegeben.
     *
     * @param[in] GPIOx: GPIOA - GPIOE */
    void initGpioClock(GPIO_t *const GPIOx)
    {
        char c = '\n';

        if(GPIOx == GPIOA)
        {
            c = 'A';
        }
        else if(GPIOx == GPIOB)
        {
            c = 'B';
        }
        else if(GPIOx == GPIOC)
        {
            c = 'C';
        }
        else if(GPIOx == GPIOD)
        {
            c = 'D';
        }
        else if(GPIOx == GPIOE)
        {
            c = 'E';
        }
        else
        {
            c = 'x';
        }

        printf("Init GPIO%c clock\n", c);
    }



    /** Takt/Clock fuer alternative Pin-Funktion bereitstellen (Dummy-Funktion)
     *
     * Die Information ueber den Aufruf wird in der Konsole ausgegeben. */
    void initAfioClock(void)
    {
        printf("Init AFIO clock\n");
    }



    /** Einzelnes oder mehrere Bits eines Ports setzen
     *
     * @param[in] GPIOx: GPIO
     * @param[in] pin: Bitmaske */
	void io_setBit(GPIO_t * const GPIOx, const IOPin_t Pin)
    {
        GPIOx->value |= Pin;
    }



    /** Einzelnes oder mehrere Bits eines Ports loeschen
     *
     * @param[in] GPIOx: GPIO
     * @param[in] pin: Bitmaske */
    void io_clearBit(GPIO_t * const GPIOx, const IOPin_t Pin)
    {
	    GPIOx->value &= ~Pin;
    }



    /** Einzelnes oder mehrere Bits eines Ports umschalten
     *
     * @param[in] GPIOx: GPIO
     * @param[in] pin: Bitmaske */
    void io_toggleBit(GPIO_t * const GPIOx, const IOPin_t Pin)
    {
	    GPIOx->value ^= Pin;
    }

#else
    typedef int this_is_no_empty_translation_unit;

#endif
