/** @file io_mock.h
 *
 * GPIO-Operationen-Mockup
 *
 * $Date: 2020-08-02 19:08:25 +0200 (So, 02 Aug 2020) $
 * $Revision: 32 $
 *
 * @date 22.02.2014 @author Nicolas */

#include "hw_config.h"

#if !defined( IO_MOCK_H ) && defined( GPIO_EMULATION )
    #define IO_MOCK_H


    #include <stdint.h>
    #include <stdbool.h>


    /** GPIO-Register */
    typedef	struct
    {
        volatile uint16_t value;
    }
    GPIO_t;

    /** GPIO-Pins (Bitmaske) */
    typedef	uint16_t IOPin_t;



    /** Bitmasken fuer GPIO-Pins */
    enum
    {
        GPIO_Pin_0   =           0x0001,
        GPIO_Pin_1   =           0x0002,
        GPIO_Pin_2   =           0x0004,
        GPIO_Pin_3   =           0x0008,
        GPIO_Pin_4   =           0x0010,
        GPIO_Pin_5   =           0x0020,
        GPIO_Pin_6   =           0x0040,
        GPIO_Pin_7   =           0x0080,
        GPIO_Pin_8   =           0x0100,
        GPIO_Pin_9   =           0x0200,
        GPIO_Pin_10  =           0x0400,
        GPIO_Pin_11  =           0x0800,
        GPIO_Pin_12  =           0x1000,
        GPIO_Pin_13  =           0x2000,
        GPIO_Pin_14  =           0x4000,
        GPIO_Pin_15  =           0x8000,
        GPIO_Pin_All =           0xFFFF,
    };


    extern GPIO_t *GPIOA;
    extern GPIO_t *GPIOB;
    extern GPIO_t *GPIOC;
    extern GPIO_t *GPIOD;
    extern GPIO_t *GPIOE;



	/* Auf dem PC sind viele Funktionen nicht inlinebar, um Debug zu erleichtern */
	void io_setBit(GPIO_t * const GPIOx, const IOPin_t Pin);
    void io_clearBit(GPIO_t * const GPIOx, const IOPin_t Pin);
    void io_toggleBit(GPIO_t * const GPIOx, const IOPin_t Pin);



    /* Einzelnes oder mehrere Bits eines Ports setzen (push-pull)
     *
     * GPIOx
     * Pin: Bitmaske
     * value: high oder low */
    static inline void io_setBitTo(GPIO_t * const GPIOx, const IOPin_t Pin, bool value)
    {
        if( value )
            GPIOx->value |= Pin;
        else
            GPIOx->value &= ~Pin;
    }



    /* Acht zusammenhaengende Bits eines Ports setzen oder loeschen
     *
     * GPIOx
     * byte  : Zielwerte
     * offset: [0..8], Offset des Bytes innerhalb des 16-Bits-Ports */
    static inline void
        io_setByte(GPIO_t * const GPIOx, const uint8_t byte, const uint8_t offset)
    {
        (void) GPIOx;
		(void) byte;
		(void) offset;
    }



    /* Acht zusammenhaengende Bits eines Ports lesen
     *
     * GPIOx
     * offset: [0..8], Offset des Bytes innerhalb des 16-Bits-Ports
     * return: gelesenes Byte */
    static inline uint8_t io_getByte(GPIO_t * const GPIOx, const uint8_t offset)
    {
        (void) GPIOx;
		(void) offset;
        return 0;
    }



    /* Logischen Wert eines Bits lesen
     * Liefert logische 1, wenn Bit gesetzt, aber nicht notwendigerweise
     * 0x01.
     *
     * GPIOx
     * bit: Bitmaske */
    static inline uint16_t io_bitIsSet(GPIO_t * const GPIOx, const IOPin_t Pin)
    {
        return GPIOx->value & Pin;
    }



    /* Einzelne Pins eines Ports per Bitmaske als frei floatenden Eingang
     * setzen.
     *
     * GPIOx
     * Pin: Bitmaske */
    static inline void io_setInput(GPIO_t const* GPIOx, const IOPin_t Pin)
	{
	    (void) GPIOx;
		(void) Pin;
	}



    /* Einzelne Pins eines Ports per Bitmaske als Push-Pull-Ausgang setzen.
     * Die Ports-Bits werden dabei nicht beeinflusst, muessen also bei
     * Bedarf vorher oder nachher einzeln gesetzt oder geloescht werden.
     *
     * GPIOx
     * Pin: Bitmaske */
    static inline void io_setOutput(GPIO_t const* GPIOx, const IOPin_t Pin)
    {
       	(void) GPIOx;
		(void) Pin;
    }



    /* Einzelne Pins eines Ports per Bitmaske als Open-Drain-Ausgang setzen.
     * Die Ports-Bits werden dabei nicht beeinflusst, muessen also bei
     * Bedarf vorher oder nachher einzeln gesetzt oder geloescht werden.
     *
     * GPIOx
     * Pin: Bitmaske */
    static inline void io_setOutputOD(GPIO_t const* GPIOx, const IOPin_t Pin)
    {
       	(void) GPIOx;
		(void) Pin;
    }



    /* Einzelnes oder mehrere Bits eines Ports setzen
     *
     * GPIOx
     * Pin: Bitmaske */
	static inline void io_setBitOD(GPIO_t * const GPIOx, const IOPin_t Pin)
    {
        GPIOx->value |= Pin;
    }



    /* Einzelnes oder mehrere Bits eines Ports loeschen
     *
     * GPIOx
     * Pin: Bitmaske */
    static inline void io_clearBitOD(GPIO_t * const GPIOx, const IOPin_t Pin)
    {
	    GPIOx->value &= ~Pin;
    }



    /* Einzelne Pins eines Ports per Bitmaske als Eingang mit Pull-Up
     * setzen.
     *
     * GPIOx
     * Pin: Bitmaske */
    static inline void io_setInputPU(GPIO_t * const GPIOx, const IOPin_t Pin)
    {
        (void) GPIOx;
		(void) Pin;
    }



    /* Hilfsfunktionen fuer Clocks */
    void initGpioClock(GPIO_t *const GPIOx);
    void initAfioClock(void);

#endif
