/** @file io_stm32f10x.c
 *
 * GPIO-Operationen fuer STM32F10x (Hilfsfunktionen fuer GPIO-Nutzung)
 *
 * benoetigt das Makro STM32F10x in der Build-Umgebung.
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 27.04.2014 @author Nicolas */


#include "io.h"

#ifdef STM32F10x

    #include "stm32f10x_rcc.h"
    #include "myassert.h"
    #include "bitmath.h"



    /** Takt/Clock fuer GPIO bereitstellen
     *
     * @param[in] GPIOx: GPIOA - GPIOE */
    void initGpioClock(GPIO_t *GPIOx)
    {
        if(GPIOx == GPIOA)
        {
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
        }
        else if(GPIOx == GPIOB)
        {
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
        }
        else if(GPIOx == GPIOC)
        {
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
        }
        else if(GPIOx == GPIOD)
        {
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE);
        }
        else if(GPIOx == GPIOE)
        {
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOE, ENABLE);
        }
        else
            error("Could not activate Clock - unknown GPIO");
    }



    /** Takt/Clock fuer alternative Pin-Funktion bereitstellen
     *
     * GPIO-Clock muss separat aktiviert werden */
    void initAfioClock(void)
    {
        RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);
    }



    /** Takt/Clock fuer Timer bereitstellen
     *
     * @param[in] TIMx : TIM1 - TIM4 */
    void initTimClock(TIM_TypeDef * TIMx)
    {
        if(TIMx == TIM1)
        {
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);
        }
        else if(TIMx == TIM2)
        {
            RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
        }
        else if(TIMx == TIM3)
        {
            RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
        }
        else if(TIMx == TIM4)
        {
            RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4,ENABLE);
        }
        else
        {
            error("Could not activate Clock - unknown Timer");
        }
    }


#else
    typedef int this_is_no_empty_translation_unit;

#endif
