/** @file Utest_i2c_scan.c
 *
 * Nutzer-interaktive Tests: Suchen von I2C-Geraeten und Darstellen auf LCD
 *
 * $Date: 2020-08-12 16:39:52 +0200 (Mi, 12 Aug 2020) $
 * $Revision: 66 $
 *
 * @date 19.10.2013 @author Nicolas */


#include "Utest_i2c_scan.h"
#include "hw_config.h"
#include "i2c_simple.h"
#include "userinterface.h"



/** Suchen von I2C-Devices und Darstellen auf dem LCD */
void Utest_i2c_scan(void)
{
	I2C_TypeDef* I2Cx = I2C_MAIN_I2Cx;
	uint8_t status;
	uint8_t nFound = 0;


    Ui_t Uim, *Ui = &Uim;
    ui_init(Ui, "Scan I2C adresses");
    ui_clear();
	ui_printf_P(PSTR("Scan I2C adresses\n"));


    /* initialize I2C peripheral */
	status = i2c_init(I2Cx);
    if( status )
    {
        ui_printf_P(PSTR("\nError initializing I2C bus\nerror code 0x%x"), status);
        goto end;
    }


	/* Check for I2C devices */
	for( ufast16_t i=0x00; i<0xFF; i+=2 )
    {
        ui_setcursor(2, 0);
		ui_printf_P(PSTR("Scanning: 0x%02X  "), i);

		status = i2c_ping(I2Cx, i);

		switch( status )
        {
		    case i2c_ok:
                ui_setcursor(3+nFound, 0);
			    ui_printf_P(PSTR("found: 0x%02X"), i);
			    nFound++;
			    break;

		    case i2c_ackTimeout:   /* NACK */
		    case i2c_noAck:        /* auch NACK */
			    break;

		    case i2c_otherTimeout: /* Busfehler: Konnte START nicht senden */
			    /* fallthru */
		    default:               /* Unterwarte Faelle */
			    i2c_reset(I2Cx);
                ui_setcursor(3+nFound, 0);
			    nFound++;
			    ui_printf_P(PSTR("Timeout=%02X,%02X \a"), i, status);
		}

		//delay_us(50000);
	}


	end:

    ui_setcursor(4+nFound, 0);
    ui_printf_P(PSTR("done."));

    ui_printf_P(PSTR("\npress key to leave\a"));
	while(!ui_getAction());
    ui_deinit(Ui);
}



