/** @file eeprom_24Cx.h
 *
 * Lesen und Schreiben eines externen EEPROMs 24C02 per I2C
 * Einstellungen werden erwartet in der i2c_config.h
 *
 * $Date: 2020-08-07 16:39:11 +0200 (Fr, 07 Aug 2020) $
 * $Revision: 54 $
 *
 * @date 02.02.2014 @author Nicolas
 */

#ifndef EEPROM_24CX_H
    #define EEPROM_24CX_H


    #include <stdint.h>


    /** Fehlercodes: EEPROM (Level-1-Peripherie) */
    typedef enum eeprom_errorcode_e
    {
        ee_ok               = 0x00,
        ee_invalidAddress   = 0x10,
        ee_noStart          = 0x20,
        ee_invalidSize      = 0x30,
  		ee_error            = 0x40, /**< interner Fehler */
  		ee_i2cerror         = 0x50,
    }
    eeprom_errorcode_t;




    uint_fast8_t ee24Cx_read_byte(uint16_t addr, uint8_t *byte);
    uint_fast8_t ee24Cx_write_byte(uint16_t addr, uint8_t byte);
    uint_fast8_t ee24Cx_read_block(uint8_t *buffer,uint16_t addr, uint16_t size);
    uint_fast8_t ee24Cx_write_block(uint8_t *buffer,uint16_t addr, uint16_t size);
    uint_fast8_t ee24Cx_update_block(uint8_t *buffer,uint16_t addr, uint16_t size);

    #ifndef AVR
        /* Der AVR hat EEPROM-Routinen mit den Namen eeprom_read_byte & Co.
         * Fuer andere Plattformen springt diese Library ein.
         * Beim AVR sind die EEPROM-Adressen Zeiger (was die Einlagerung mit
         * EEMEM erlaubt), hier sind die Adressen einfach Integer. */
        #define eeprom_read_byte    ee24Cx_read_byte
        #define eeprom_write_byte   ee24Cx_write_byte
        #define eeprom_read_block   ee24Cx_read_block
        #define eeprom_write_block  ee24Cx_write_block
        #define eeprom_update_block ee24Cx_update_block
    #endif
#endif
