/** @file i2c_simple_mock.c
 *
 * Einfache Variante von I2C-Routinen
 *
 * Primitiver Stub, der den Aufruf ueber die Kommandozeile protokolliert und immer OK zurueckgibt
 *
 * @date 10.04.15 @author Nicolas */

#include "i2c_simple.h"
#include "hw_config.h"
#include <stdio.h>
#include "i2c.h"



#if defined( I2C_MOCK ) || DOXYGEN


    /** I2C (Mockup): Initialisierung
     *
     * @param[in] I2Cx: dummy
     * @return i2c_ok */
    i2c_errorcode_t  i2c_init(I2C_TypeDef* I2Cx)
    {
        (void) I2Cx;
        printf("I2C_init - return i2c_ok\n");
        return i2c_ok;
    }



    /** I2C (Mockup): Deinitialisierung
     *
     * @param[in] I2Cx: Dummy */
    ufast8_t i2c_deinit(I2C_TypeDef* I2Cx)
    {
        (void) I2Cx;
        printf("I2C_init - return i2c_ok\n");
        return i2c_ok;
    }



    /** I2C (Mockup): Reset/Freitakten
     *
     * @param[in] I2Cx: dummy
     * @return i2c_ok */
    ufast8_t i2c_mock_reset(I2C_TypeDef* I2Cx)
    {
        (void) I2Cx;
        printf("I2C_reset - return i2c_ok\n");
        return i2c_ok;
    }



    /** I2C (Mockup): Busy-Status abwarten, muss vor jedem Start ausgefuehrt werden
     * @param[in] I2Cx: dummy
     * @return i2c_ok */
    uint_fast8_t I2C_waitbusy(I2C_TypeDef* I2Cx)
    {
        printf("I2C_waitbusy 0x%x\n",(int) (intptr_t) I2Cx);
        return i2c_ok;
    }



    /** I2C (Mockup): Start condition und R/W-Bit senden (Dummy)
     * @param[in] I2Cx: I2C1 oder I2C2 (dummy)
     * @param[in] address  : 7-Bit-Adresse linksbuendig
     * @param[in] direction: I2C_Direction_Tranmitter (Master+Senden) oder
     *            I2C_Direction_Receiver (Master+Empfangen)
     * @return i2c_ok */
    uint_fast8_t I2C_start(I2C_TypeDef* I2Cx, uint8_t address, uint8_t direction)
    {
        printf("I2C_start 0x%x -> %x,%i \n",(int) (intptr_t) I2Cx, address, direction);
        return i2c_ok;
    }



    /** I2C (Mockup): Ein Byte an Slave senden
     * @param[in] I2Cx: I2C1 oder I2C2 (dummy)
     * @param[in] data: Byte
     * @return i2c_ok */
    uint_fast8_t I2C_write(I2C_TypeDef* I2Cx, uint8_t data)
    {
        printf("I2C_write 0x%x -> %x\n",(int) (intptr_t) I2Cx,data);
        return i2c_ok;
    }



    /** I2C (Mockup): Ein Byte vom Slave lesen und mit ACK besttigen = ein weiteres Byte anfordern
     * @param[in] I2Cx: I2C1 oder I2C2 (Dummy)
     * @param[out] data: Byte
     * @return i2c_ok */
    uint_fast8_t I2C_read_ack(I2C_TypeDef* I2Cx, uint8_t *data)
    {
        *data = 0;
        printf("I2C_read_ack 0x%x \n",(int) (intptr_t) I2Cx);
        return i2c_ok;
    }



    /** I2C (Mockup): Ein Byte vom Slave lesen und mit NACK besttigen = kein weiteres Byte anfordern
     * @param[in] I2Cx: I2C1 oder I2C2 (Dummy)
     * @param[out] data: Byte
     * @return i2c_ok */
    uint_fast8_t I2C_read_nack(I2C_TypeDef* I2Cx, uint8_t *data)
    {
        *data = 0;
        printf("I2C_read_nack 0x%x \n",(int) (intptr_t) I2Cx);
        return i2c_ok;
    }



    /** I2C (Mockup): STOP condition senden und Bus freigeben
     * @param[in] I2Cx: I2C1 oder I2C2 (Dummy)
     * @return i2c_stop */
    uint_fast8_t I2C_stop(I2C_TypeDef* I2Cx)
    {
        printf("I2C_stop 0x%x \n",(int) (intptr_t) I2Cx);
        return i2c_stop;
    }

#else
    typedef int this_is_no_empty_translation_unit;

#endif
