/*
 * $Date: 2020-08-01 09:01:46 +0200 (Sa, 01 Aug 2020) $
 * $Revision: 27 $
 */

#include "bitmath.h"
#include "sw_config.h"



#if defined( INCLUDE_UNIT_TESTS )

    #include "unity_fixture.h"


    TEST_GROUP(bitmath);
    TEST_SETUP(bitmath) {}
    TEST_TEAR_DOWN(bitmath) {}


    TEST_GROUP_RUNNER(bitmath)
    {
        RUN_TEST_CASE(bitmath, areAllBitsSet);
        RUN_TEST_CASE(bitmath, areAnyBitsSet);
        RUN_TEST_CASE(bitmath, nullArgument);
        RUN_TEST_CASE(bitmath, bytes4bits);
        RUN_TEST_CASE(bitmath, byteReverseBitOrder);
        RUN_TEST_CASE(bitmath, areBitsEqual);
    }




    TEST(bitmath, areAllBitsSet)
    {
        TEST_ASSERT_EQUAL( true, areAllBitsSet(0x3, 0x1) );
        TEST_ASSERT_EQUAL( false, areAllBitsSet(0x3, 0x4) );
        TEST_ASSERT_EQUAL( false, areAllBitsSet(0x3, ~0x3) );

    }

    TEST(bitmath, nullArgument)
    {
        prepareErrorCondition();
        if( TEST_PROTECT() )
        {
            // Funktionen sind fuer zweiter Parameter 0 komplett sinnlos
            areAllBitsSet(0x0, 0x0);
        }
        TEST_ASSERT_TRUE( isErrorConditionTriggered() );
        clearErrorCondition();

        prepareErrorCondition();
        if( TEST_PROTECT() )
        {
            // Funktionen sind fuer zweiter Parameter 0 komplett sinnlos
            areAnyBitsSet(0x0, 0x0);
        }
        TEST_ASSERT_TRUE( isErrorConditionTriggered() );
        clearErrorCondition();
    }



    TEST(bitmath, areAnyBitsSet)
    {
        TEST_ASSERT_EQUAL( true, areAnyBitsSet(0x3, 0x1) );
    }


    TEST(bitmath, areBitsEqual)
    {
        TEST_ASSERT_EQUAL( true, areBitsEqual(0x3, 0x3, 0x3) );
        TEST_ASSERT_EQUAL( true, areBitsEqual(0x3, ~0x3, 0x0) );
        TEST_ASSERT_EQUAL( false, areBitsEqual(0x3, ~0x3, 0x1) );
        TEST_ASSERT_EQUAL( true, areBitsEqual(0x3,  0x1, 0x1) );
    }





    TEST(bitmath, bytes4bits)
    {
        TEST_ASSERT_EQUAL( 0, bytes4bits(0) );
        TEST_ASSERT_EQUAL( 1, bytes4bits(8) );
        TEST_ASSERT_EQUAL( 2, bytes4bits(9) );
        TEST_ASSERT_EQUAL( 2, bytes4bits(16) );
        TEST_ASSERT_EQUAL( 3, bytes4bits(17) );
        TEST_ASSERT_EQUAL( 4, bytes4bits(32) );
        TEST_ASSERT_EQUAL( 5, bytes4bits(33) );
    }


    TEST(bitmath, byteReverseBitOrder)
    {
        TEST_ASSERT_EQUAL_HEX( 0,    byteReverseBitOrder(0) );
        TEST_ASSERT_EQUAL_HEX( 0x80, byteReverseBitOrder(1) );
        TEST_ASSERT_EQUAL_HEX( 0x40, byteReverseBitOrder(2) );
        TEST_ASSERT_EQUAL_HEX( 0xFF, byteReverseBitOrder(0xFF) );
        TEST_ASSERT_EQUAL_HEX( 0xAA, byteReverseBitOrder(0x55) );
    }

#else
    typedef int this_is_no_empty_translation_unit;

#endif

