/*
 * $Date: 2020-08-01 09:01:46 +0200 (Sa, 01 Aug 2020) $
 * $Revision: 27 $
 */

#include "crc.h"
#include "sw_config.h"



#if defined( INCLUDE_UNIT_TESTS )

    #include "unity_fixture.h"

    TEST_GROUP(crc);
    TEST_SETUP(crc) {}
    TEST_TEAR_DOWN(crc) {}

    TEST_GROUP_RUNNER(crc)
    {
        RUN_TEST_CASE(crc, power);
        RUN_TEST_CASE(crc, testValues);
        RUN_TEST_CASE(crc, testZero);
        RUN_TEST_CASE(crc, testManipulated);
    }


    TEST(crc, power)
    {
        #if 0
        TEST_ASSERT_EQUAL( 10, decibel_dB2pow(10) );
        TEST_ASSERT_EQUAL( 2, decibel_dB2pow(3) );
        TEST_ASSERT_EQUAL( 4, decibel_dB2pow(6) );
        TEST_ASSERT_EQUAL( 79, decibel_dB2pow(19) );
        TEST_ASSERT_EQUAL( 8, decibel_dB2pow(9) );
        TEST_ASSERT_EQUAL( 1000000, decibel_dB2pow(60) );
        TEST_ASSERT_EQUAL( 0, decibel_dB2pow(97) );
        #endif
    }


    TEST(crc, testValues)
    {
        uint8_t buffer[12]="Hello";
        uint16_t crc1;

        // CRC16 berechnen
        crc1 = crc16_calc(buffer, 5);
        TEST_ASSERT_EQUAL_HEX( 0xF377, crc1 );
    }


    TEST(crc, testZero)
    {
        uint8_t buffer[12]={10, 2, 3, 4, 57, 6, 7, 8, 9, 10};
        uint16_t crc1, crc2;

        // CRC16 berechnen
        crc1 = crc16_calc(buffer, 10);

        // Ergebnis an Datensatz anhaengen
        buffer[11] = crc1>>8;
        buffer[10] = crc1;

        // CRC von erweitertem Datensatz muss Null ergeben.
        crc2 = crc16_calc(buffer,12);
        TEST_ASSERT_EQUAL_HEX( 0, crc2 );
    }


    TEST(crc, testManipulated)
    {
        uint8_t buffer[12]={10, 2, 3, 4, 57, 6, 7, 8, 9, 10};
        uint16_t crc1, crc2;

        // CRC16 berechnen
        crc1 = crc16_calc(buffer, 10);

        // Ergebnis an Datensatz anhaengen
        buffer[11] = crc1>>8;
        buffer[10] = crc1;

        // Datensatz aendern
        buffer[3] += 1;

        // CRC von erweitertem, manupiliertem Datensatz darf nicht Null ergeben.
        crc2 = crc16_calc(buffer,12);
        TEST_ASSERT_NOT_EQUAL( 0, crc2 );
    }



#else
    typedef int this_is_no_empty_translation_unit;

#endif


