/*
 * $Date: 2020-08-01 09:01:46 +0200 (Sa, 01 Aug 2020) $
 * $Revision: 27 $
 */

#include "sw_config.h"



#if defined( INCLUDE_UNIT_TESTS )

    #include "intmath.h"
    #include "unity_fixture.h"
    #include "myassert.h"



    TEST_GROUP(intmath);
    TEST_SETUP(intmath) {}
    TEST_TEAR_DOWN(intmath) {}


    TEST_GROUP_RUNNER(intmath)
    {
        RUN_TEST_CASE(intmath, llog2);
        RUN_TEST_CASE(intmath, clamp_s32);
        RUN_TEST_CASE(intmath, satadd_s32);
        RUN_TEST_CASE(intmath, satadd_u32);
        RUN_TEST_CASE(intmath, satadd_s16);
        RUN_TEST_CASE(intmath, satadd_s8);
        RUN_TEST_CASE(intmath, satakku_u8);
        RUN_TEST_CASE(intmath, satmul_s8);
        RUN_TEST_CASE(intmath, ispoweroftwo);
        RUN_TEST_CASE(intmath, ISPOWEROFTWO);
        RUN_TEST_CASE(intmath, gcd_s32);
        RUN_TEST_CASE(intmath, lcm_s32);
        RUN_TEST_CASE(intmath, productFits_s32);
        RUN_TEST_CASE(intmath, inIntervall_s64);
        RUN_TEST_CASE(intmath, div_s64);
        RUN_TEST_CASE(intmath, expandInt_s32);
    }


    TEST(intmath, llog2)
    {
        TEST_ASSERT_EQUAL( 1, log2_u32(2) );
        TEST_ASSERT_EQUAL( 0, log2_u32(1) );
        TEST_ASSERT_EQUAL( 2, log2_u32(4) );
        TEST_ASSERT_EQUAL( 31, log2_u32((uint32_t) INT32_MAX+1UL) );
        TEST_ASSERT_EQUAL( 31, log2_u32(UINT32_MAX) );
    }


    TEST(intmath, clamp_s32)
    {
        int32_t x=10;
        int isclamped;
        isclamped = clamp_s32(&x, 1, 5);
        TEST_ASSERT_EQUAL( 5, x );
        TEST_ASSERT_EQUAL( 1, isclamped );

        x = 0;
        isclamped = clamp_s32(&x, 1, 5);
        TEST_ASSERT_EQUAL( 1, x );
        TEST_ASSERT_EQUAL( -1, isclamped );

        x = 4;
        isclamped = clamp_s32(&x, 1, 5);
        TEST_ASSERT_EQUAL( 4, x );
        TEST_ASSERT_EQUAL( 0, isclamped );

        x = -14;
        isclamped = clamp_s32(&x, -10, 10);
        TEST_ASSERT_EQUAL( -10, x );
        TEST_ASSERT_EQUAL( -1, isclamped );
    }


    TEST(intmath, satadd_s32)
    {

        TEST_ASSERT_EQUAL( INT32_MAX, satadd_s32(INT32_MAX, 1) );
        TEST_ASSERT_EQUAL( INT32_MAX, satadd_s32(2, INT32_MAX) );
        TEST_ASSERT_EQUAL( INT32_MAX, satadd_s32(INT32_MAX-1, 1) );
        TEST_ASSERT_EQUAL( -1, satadd_s32(INT32_MIN, INT32_MAX) );
        TEST_ASSERT_EQUAL( INT32_MIN, satadd_s32(INT32_MIN, -4) );
        TEST_ASSERT_EQUAL( -8, satadd_s32(8, -16) );
        TEST_ASSERT_EQUAL( -8, satadd_s32(-16, 8) );
        TEST_ASSERT_EQUAL( -15, satadd_s32(-8, -7) );
    }


    TEST(intmath, satadd_u32)
    {
        TEST_ASSERT_EQUAL( 5, satadd_u32(3, 2) );
        TEST_ASSERT_EQUAL( UINT32_MAX, satadd_u32(UINT32_MAX, 1) );
    }


    TEST(intmath, satadd_s16)
    {
        // satadd_s16() wird einzeln getestet, da direkter Aufruf des Makros CLAMP und
        // wegen des exotischen Datentype int_fast24_t
        TEST_ASSERT_EQUAL( INT16_MAX, satadd_s16(INT16_MAX, 1) );
        TEST_ASSERT_EQUAL( INT16_MAX, satadd_s16(2, INT16_MAX) );
        TEST_ASSERT_EQUAL( INT16_MAX, satadd_s16(INT16_MAX-1, 1) );
        TEST_ASSERT_EQUAL( -1, satadd_s16(INT16_MIN, INT16_MAX) );
        TEST_ASSERT_EQUAL( INT16_MIN, satadd_s16(INT16_MIN, -4) );
        TEST_ASSERT_EQUAL( -8, satadd_s16(8, -16) );
        TEST_ASSERT_EQUAL( -8, satadd_s16(-16, 8) );
        TEST_ASSERT_EQUAL( -15, satadd_s16(-8, -7) );
    }


    TEST(intmath, satadd_s8)
    {
        TEST_ASSERT_EQUAL( 127, satadd_s8(100, 111) );
        TEST_ASSERT_EQUAL( -128, satadd_s8(-100, -30) );
        TEST_ASSERT_EQUAL( 28, satadd_s8(13, 15) );
    }


    TEST(intmath, satakku_u8)
    {
        TEST_ASSERT_EQUAL( 255, satakku_u8(130, 127) );
        TEST_ASSERT_EQUAL( 0, satakku_u8(100, -127) );
        TEST_ASSERT_EQUAL( 28, satakku_u8(13, 15) );
    }


    TEST(intmath, satmul_s8)
    {
        TEST_ASSERT_EQUAL( INT8_MAX, satmul_s8(100, 111) );
        TEST_ASSERT_EQUAL( INT8_MAX, satmul_s8(INT8_MAX, INT8_MAX));
        TEST_ASSERT_EQUAL( INT8_MAX, satmul_s8(INT8_MIN, INT8_MIN));
        TEST_ASSERT_EQUAL( INT8_MIN, satmul_s8(INT8_MAX, INT8_MIN));
        TEST_ASSERT_EQUAL( 18, satmul_s8(6, 3));
        TEST_ASSERT_EQUAL( -18, satmul_s8(6, -3));
    }


    TEST(intmath, ispoweroftwo)
    {
        TEST_ASSERT_EQUAL( true, ispoweroftwo(1) );
        TEST_ASSERT_EQUAL( true, ispoweroftwo(2) );
        TEST_ASSERT_EQUAL( false, ispoweroftwo(-2) );
    }


    TEST(intmath, ISPOWEROFTWO)
    {
        TEST_ASSERT_EQUAL( false, ISPOWEROFTWO(0) );
        TEST_ASSERT_EQUAL( true, ISPOWEROFTWO(1) );
        TEST_ASSERT_EQUAL( true, ISPOWEROFTWO(2) );
        TEST_ASSERT_EQUAL( false, ISPOWEROFTWO(-2) );
    }


    TEST(intmath, gcd_s32)
    {
        // Bei gemischt-negativen Zahlen ist das Vorzeichen des Ergebnisses nicht intuitiv
        TEST_ASSERT_EQUAL( 1, gcd_s32(3L, 1L ) );
        TEST_ASSERT_EQUAL( 1, ABS(gcd_s32(3L, -1L )) );
        TEST_ASSERT_EQUAL( 1, ABS(gcd_s32(-1L, 3L )) );
        TEST_ASSERT_EQUAL( 1, ABS(gcd_s32(1L, -3L )) );
        TEST_ASSERT_EQUAL( 3, ABS(gcd_s32(-6L, -3L )) );
        TEST_ASSERT_EQUAL( LONG_MAX, gcd_s32(LONG_MAX, LONG_MAX ) );
        TEST_ASSERT_EQUAL( 1, gcd_s32(1, LONG_MAX ) );
        TEST_ASSERT_EQUAL(  1, gcd_s32(LONG_MIN+1, 1) );
        TEST_ASSERT_EQUAL(  1, gcd_s32(LONG_MIN, 1) );
        TEST_ASSERT_EQUAL(  1, gcd_s32(1, LONG_MIN ) );
        TEST_ASSERT_EQUAL( LONG_MIN, gcd_s32(LONG_MIN, LONG_MIN ) );
        TEST_ASSERT_EQUAL( -1, gcd_s32(LONG_MIN, LONG_MAX ) );
        TEST_ASSERT_EQUAL( 0, gcd_s32(0, 0 ) );
        TEST_ASSERT_EQUAL( 10, gcd_s32(10, 0 ) );
        TEST_ASSERT_EQUAL( 10, gcd_s32(0, 10 ) );
    }


    TEST(intmath, lcm_s32)
    {
        TEST_ASSERT_EQUAL( 3, lcm_s32(3, 1) );
        TEST_ASSERT_EQUAL( 3, lcm_s32(3, -1) );
        TEST_ASSERT_EQUAL( -3, lcm_s32(-3, 1) );
        TEST_ASSERT_EQUAL( -3, lcm_s32(-3, -1) );
        TEST_ASSERT_EQUAL( INT32_MAX, lcm_s32(INT32_MAX, 3) );
        TEST_ASSERT_EQUAL( INT32_MIN, lcm_s32(INT32_MIN, 3) );
        TEST_ASSERT_EQUAL( INT32_MAX, lcm_s32(INT32_MAX, INT32_MIN) );
        TEST_ASSERT_EQUAL( INT32_MIN, lcm_s32(INT32_MIN, INT32_MIN) );
    }


    TEST(intmath, productFits_s32)
    {
        TEST_ASSERT_EQUAL( true, productFits_s32(3, 1) );
        TEST_ASSERT_EQUAL( false, productFits_s32(INT32_MAX/2, 3) );
        TEST_ASSERT_EQUAL( true, productFits_s32(UINT16_MAX, INT16_MAX) );
        TEST_ASSERT_EQUAL( false, productFits_s32(UINT16_MAX, UINT16_MAX) );
        TEST_ASSERT_EQUAL( true, productFits_s32(UINT16_MAX, INT16_MAX+1) );
    }



    TEST(intmath, inIntervall_s64)
    {
        TEST_ASSERT_EQUAL( true, inIntervall_s64(0, 1, -1) );
        TEST_ASSERT_EQUAL( true, inIntervall_s64(0, -1, 1) );
        TEST_ASSERT_EQUAL( false, inIntervall_s64(2, -1, 1) );
        TEST_ASSERT_EQUAL( true,  inIntervall_s64(INT64_MAX, INT64_MAX-1, INT64_MAX) );
        TEST_ASSERT_EQUAL( false,  inIntervall_s64(INT64_MAX, INT64_MAX-1, INT64_MIN) );
        TEST_ASSERT_EQUAL( false,  inIntervall_s64(INT64_MIN, INT64_MIN+1, INT64_MAX) );
        TEST_ASSERT_EQUAL( true,   inIntervall_s64(INT64_MIN, INT64_MIN, INT64_MAX) );
    }




    TEST(intmath, div_s64)
    {


        TEST_ASSERT_EQUAL( 2, div_s64(5, 3) );
        TEST_ASSERT_EQUAL( -2, div_s64(5, -3) );
        TEST_ASSERT_EQUAL( -2, div_s64(-5, 3) );

        prepareErrorCondition();
        if( TEST_PROTECT() )
        {
            div_s64(0, 0);
        }
        TEST_ASSERT_TRUE( isErrorConditionTriggered() );
        clearErrorCondition();

        prepareErrorCondition();
        if( TEST_PROTECT() )
        {
            div_s64(0, 0);
        }
        TEST_ASSERT_TRUE( isErrorConditionTriggered() );
        clearErrorCondition();

        prepareErrorCondition();
        if( TEST_PROTECT() )
        {
            div_s64(INT64_MAX, 2);
        }
        TEST_ASSERT_TRUE( isErrorConditionTriggered() );
        clearErrorCondition();

        TEST_ASSERT_EQUAL(1, div_s64(INT64_MAX/2, INT64_MAX/2));

        prepareErrorCondition();
        if( TEST_PROTECT() )
        {
            div_s64(INT64_MIN, 2);
        }
        TEST_ASSERT_TRUE( isErrorConditionTriggered() );
        clearErrorCondition();
    }



    TEST(intmath, expandInt_s32)
    {
        prepareErrorCondition();
        if( TEST_PROTECT() )
        {
            expandInt_s32( -1 & ((1U<<24)-1), 33 );
        }
        TEST_ASSERT_TRUE( isErrorConditionTriggered() );
        clearErrorCondition();

        uint32_t mask =  ((1U<<24)-1);
        TEST_ASSERT_EQUAL( -1, expandInt_s32( -1 & mask, 24 ) );
        TEST_ASSERT_EQUAL( -1, expandInt_s32( -1 , 32 ) );
        TEST_ASSERT_EQUAL( INT16_MIN, expandInt_s32( INT16_MIN , 16 ) );
    }

#else
    typedef int this_is_no_empty_translation_unit;

#endif
