/** @file crc.c
 *
 * Berechnung von CRC16 mit fixem Generatorpolynom
 *
 * Berechnung eines:
 *  - CRC16 mit dem Generatorpolynom 0xA001
 *  - CRC4 mit dem Generatorpolynom 0xD
 *
 * @date 08.03.2014 @author Nicolas */


#include "crc.h"
#include <stdint.h>
#ifdef AVR
    #include <util/crc16.h>
#endif




#ifdef AVR

    /** Die AVR-Libc hat eine ASM-Implementierung eingebaut */
    static uint16_t crc16_update(uint16_t crc, uint8_t a)
    {
    	return _crc16_update(crc, a);
    }

#else

    /** Fuer andere Plattformen wird die Implementierung aus der Doku der AVR-libc verwendet */
    static uint16_t crc16_update(uint16_t crc, uint8_t a)
    {
    	uint_fast8_t i;

    	crc ^= a;
    	for( i = 0; i < 8; ++i )
    	{
    		if (crc & 1)
    			crc = (crc >> 1) ^ 0xA001;
    		else
    			crc = (crc >> 1);
    	}

    	return crc;
    }

#endif


/** Berechnung CRC16 mit dem Generatorpolynom 0xA001
 *
 * @param[in] buffer: Byte-Array, ueber das die Pruefsumme gebildet werden soll
 * @param[in] buffersize: Laenge des Byte-Arrays
 * @return Pruefsumme */
uint16_t crc16_calc(const uint8_t *buffer, uint8_t buffersize)
{
  	uint16_t crc = 0xFFFF;
  	uint8_t i;
  	for( i=0; i<buffersize; i++ )
    {
  		crc = crc16_update(crc, *buffer++);
  	}
  	return crc;
}



