/** @file intmath.c
 *
 * Einfache Funktionen fuer die Integer-Mathematik
 *
 * z.B.:
 *   - Saettigende Addition, Subtraktion und Multiplikation
 *   - Kleinstes gemeinsames Vielfaches
 *   - Groesster gemeinsamer Teiler
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 08.03.2014 @author Nicolas
 */


#include "intmath.h"
#include "myassert.h"
#include <limits.h>



/** Testen, ob Produkt zweier Variablen in 32 Bit passt
 * @param[in] a, b: Summanden
 * @return Ergebnis passt in 32 Bit true|false */
bool productFits_s32(int32_t a, int32_t b)
{
    /* Multiplikationsergebnis bei vorzeichenloser Zahl ist identisch */
    int64_t r = (int64_t) a * b;
    if( r < INT32_MIN || r > INT32_MAX )
    {
        return false;
    }
    else
    {
        return true;
    }
}



/** Groesster gemeinsamer Teiler/Greatest common divisor
 *
 * Algorithmus aus den Weiten des Internets, da builtin __gcd() nicht verfuegbar scheint
 *
 * @param[a, b] Zwei Zahlen
 * @return Groesster gemeinsamer Teiler */
int32_t gcd_s32(int32_t a, int32_t b)
{
    int32_t c;
    while ( a != 0 )
    {
        c = a; a = b%a;  b = c;
    }
    return b;
}



/** Groesster gemeinsamer Teiler/Greatest common divisor
 *
 * Algorithmus aus den Weiten des Internets, da builtin __gcd() nicht verfuegbar scheint
 *
 * @param[a, b] Zwei Zahlen
 * @return Groesster gemeinsamer Teiler */
int64_t gcd_s64(int64_t a, int64_t b)
{
    int64_t c;
    while ( a != 0 )
    {
        c = a; a = b%a;  b = c;
    }
    return b;
}



/** Kleinstes gemeinsames Vielfaches / Least common multiple,
 *
 * Der Wertebereich des kleinsten gemeinsamen Vielfachen kann den der Eingabewerte ueberschreiten.
 *
 * @param[in] a, b: Zwei Zahlen
 * @return Kgv */
int32_t lcm_s32(int32_t a, int32_t b )
{
    int64_t lcm = (int64_t) a * b/gcd_s32(a, b);
    if( lcm > INT32_MAX )
    {
        return INT32_MAX;
    }
    else if( lcm < INT32_MIN )
    {
        return INT32_MIN;
    }
    else
    {
        return lcm;
    }
}



/** Kleinstes gemeinsames Vielfaches / Least common multiple,
 *
 * Der Wertebereich des kleinsten gemeinsamen Vielfachen kann den der Eingabewerte ueberschreiten.
 *
 * @param[in] a, b: Zwei Zahlen
 * @return Kgv */
int32_t lcm_s16(int16_t a, int16_t b )
{
    return (int32_t) a * b/gcd_s32(a, b);
}
