/*
 * $Date: 2020-08-11 18:40:35 +0200 (Di, 11 Aug 2020) $
 * $Revision: 58 $
 */


/* Build-Auswahl fuer Tests */
#include "sw_config.h"



#if defined( INCLUDE_UNIT_TESTS )


    #include "menu_color.h"
    #include "unity_fixture.h"
    #include "menue_templates.h"
    #include "menu_color_private_item.h"
    #include "text_window.h"
    #include "u8g2_fontdata.h"


    extern Textwindowdump_t Dump;


    TEST_GROUP(menu_color);
    TEST_SETUP(menu_color)
    {
        /* Textfont sichern, damit bei Modultests auf der Zielplattform nicht die komplette
           Ausgabe geschrottet wird. */
        Dump =  textwindow_dumpSettings();
        textwindow_setFont(font_largestFontAvailable);
    }


    TEST_TEAR_DOWN(menu_color)
    {
        textwindow_restoreSettings(Dump);
    }

    TEST_GROUP_RUNNER(menu_color)
    {
        RUN_TEST_CASE(menu_color, menuitem_arraylen);
        RUN_TEST_CASE(menu_color, emptyConfig);
        RUN_TEST_CASE(menu_color, almostEmptyConfig);
        RUN_TEST_CASE(menu_color, simpleMenuReturn);
        RUN_TEST_CASE(menu_color, nullString);
    }

    static int mockIncrement(void)
    {
        return 0;
    }

    static bool mockAction(void)
    {
        return true;
    }





    /* Menucontrol, die Tastendruecke fuer eine Rueckkehr emuliert */
    constflash Menucontrol_t Menucontrol_unittest =
    {
        .pIncrFcn = mockIncrement,
        .pActnFcn = mockAction,
        .pInitFcn = NULL,
    };


    /* Einstellungen: Einfaches Standard-Menue */
    constflash Menuconfig_t MenuConfig_unittest =
    {
        .Dimension = &Menudimension_standard,
        .Color = &ColorScheme_dark,
        .Control   = &Menucontrol_unittest,
        .headtext = NULL,
        .iconFont = NULL,
    };


    TEST(menu_color, menuitem_arraylen)
    {
        Menuitem_t MenuData[2];
        MenuData[1] = (Menuitem_t) { "", .All = {endmarker} };
        MenuData[0] = (Menuitem_t) { "Hallo" , .All     ={ exitMenu }        };


        TEST_ASSERT_EQUAL( 1, menuitem_arraylen(MenuData));

        MenuData[0] = (Menuitem_t) { NULL, .All     ={ exitMenu }        };
        TEST_ASSERT_EQUAL( 0, menuitem_arraylen(MenuData));

        MenuData[0] = (Menuitem_t) { "", .All     ={ exitMenu }        };
        TEST_ASSERT_EQUAL( 0, menuitem_arraylen(MenuData));
    }



    TEST(menu_color, emptyConfig)
    {
        Menuitem_t MenuData[2];
        MenuData[1] = (Menuitem_t) { "", .All = {endmarker} };
        MenuData[0] = (Menuitem_t) { "Hallo" , .All     ={ exitMenu }        };

        prepareErrorCondition();
        if( TEST_PROTECT() )
        {
            menu_standard(MenuData, NULL, 0);
        }
        TEST_ASSERT_TRUE( isErrorConditionTriggered() );
        clearErrorCondition();
    }



    TEST(menu_color, simpleMenuReturn)
    {
        Menuitem_t MenuData[2];
        MenuData[1] = (Menuitem_t) { "", .All = {endmarker} };
        MenuData[0] = (Menuitem_t) { "Hallo" , .All     ={ exitMenu }        };

        Menuconfig_t MenuConfig = MenuConfig_unittest;
        MenuConfig.Control = &Menucontrol_unittest;
        int r = menu_standard(MenuData, &MenuConfig, 0);
        TEST_ASSERT_EQUAL( 0, r );
    }


    TEST(menu_color, almostEmptyConfig)
    {
        printf("Start aEC\n");
        Menuitem_t MenuData[2];
        MenuData[1] = (Menuitem_t) { "", .All = {endmarker} };
        MenuData[0] = (Menuitem_t) { "Hallo" , .All     ={ exitMenu }        };

        Menuconfig_t MenuConfig = MenuConfig_unittest;
        MenuConfig.headtext = NULL;
        MenuConfig.iconFont = NULL;
        MenuConfig.iconLeft = 0;
        MenuConfig.iconRight = 0;
        int r = menu_standard(MenuData, &MenuConfig, 0);
        TEST_ASSERT_EQUAL( 0, r );
    }



    TEST(menu_color, nullString)
    {
        Menuitem_t MenuData[1];

        // NULL-Pointer-String (wird als Endmarker erkannt)
        MenuData[0] = (Menuitem_t) { NULL , .All     ={ exitMenu }        };

        Menuconfig_t MenuConfig = MenuConfig_unittest;
        MenuConfig.Control = &Menucontrol_unittest;

        int r = menu_standard(MenuData, &MenuConfig, 0 );
        TEST_ASSERT_EQUAL( 0, r );
    }



#else
    typedef int this_is_no_empty_translation_unit;
#endif
