/** @file menu_color.c
 *
 * Menusystem (Darstellung eines Menues oder Popup-Fensters)
 *
 * $Date: 2020-08-11 18:40:35 +0200 (Di, 11 Aug 2020) $
 * $Revision: 58 $
 *
 * @date 10.07.2013 @author Nicolas  */


#include "menu_color.h"
#include "menu_color_private_color.h"
#include "menu_color_private_control.h"
#include "menu_color_private_geometry.h"
#include "menu_color_private_draw.h"
#include "menu_color_private_item.h"


#include "sw_config.h"
#include "text_window.h"




/* Definitionen in sw_config.h pruefen */
#if !( defined( MENU_INKREMENT_DOWN ) || defined( MENU_INKREMENT_UP ) )
    #error "Missing preprocessor Symbol: MENU_INKREMENT_DOWN or MENU_INKREMENT_UP"
#endif




/* ************************************************************************************ */
/* Menu: Auswahl- und Manipulationsbereich                                              */
/* ************************************************************************************ */



/** Menue-Hauptfunktion
 * Das Menue kann zu startende Funktionen, anzuzeigenden Text und zu aendernde Variablen
 * erhalten. Meist ist der Rueckgabewert nebensaechlich.
 *
 * @param[in] Menudata: Menue-Eintraege
 * @param[in] Menuconfig: Menue-Einstellungen
 * @param[in] selectedItem: Index des beim Start ausgewaehlten Menupunkts [0...size(Menudata)]
 * @return Index des zuletzt beim verlassen ausgewaehlten Menupunkts */
ufast8_t menu_standard(constflash Menuitem_t Menudata[], constflash Menuconfig_t *MCfg, uint8_t selectedItem)
{
    assert( MCfg != NULL );

    Textwindowdump_t Textwindowdump = textwindow_dumpSettings();

    /* Text-Fenster initialisieren */
    assert( MCfg->Dimension != NULL );
    text_setWindow(menu_private_getTextArea(*(MCfg->Dimension)));

    assert( MCfg->Dimension != NULL );
    textwindow_setColors(getTextWindowColor(*(MCfg->Color)));


    menucontrol_init(MCfg->Control);

    /* Der dickste Brocken: */
    Menudrawdata_t *MDrawData = menudraw_init(Menudata, MCfg);


    /* Externe Start-Funktion ausfuehren */
    menucontrol_executeExternalInitFcn(MCfg->Control);


    /* Menu-Item-bezogene Daten */
    uint8_t nItem = menuitem_arraylen(Menudata);
    assert( selectedItem <= nItem );


    /* Menumodus */
    enum
    {
        select,  /* Menueauswahl */
        action   /* Variable bearbeiten/Funktion ausfuehren */
    }
    encoderMode = select;


    /* Menue-Hauptschleife */
    while( 1 )
    {
        /* Wiederholende externe Funktion ausfuehren */
        menucontrol_executeExternalLoopFcn(MCfg->Control);

		/* Darstellung Men */
        menudraw_update(MDrawData, selectedItem);

		/* Auf Tastendruck reagieren und Eimgabemodus umschalten */
		if( menucontrol_getAction(MCfg->Control) )
        {
			switch( encoderMode )
            {
			    case select:
			        encoderMode = action;
				    break;
		        case action:
			        encoderMode = select;
			}
		}


        /* Vorgehen nach Eingabemodus
		   Drehgeber auswerten (nur Aenderungen) */
        ifast8_t enc_diff = menucontrol_getIncrement(MCfg->Control);

		switch( encoderMode )
        {
		    case select:
			    /* Drehgeber waehlt Menueeintrag */
                #if defined( MENU_INKREMENT_UP )
                    /* enc_diff negativ, damit Linksdrehung nach unten geht */
                    enc_diff = -enc_diff;
                #endif
                selectedItem = satakku_u8(selectedItem, enc_diff);
                clamp_u8(&(selectedItem), 0, nItem-1);

		        break;

		    case action:
                {
                    /* Funktionsaufruf oder bearbeiten einer Variable */
                    menuitem_caller_action_t cmd = menuitem_execute(Menudata[selectedItem], enc_diff);

                    switch( cmd )
                    {
						case doNothing:
							break;

						case doReset:
                            menucontrol_reset(MCfg->Control);
                            menudraw_reset(MDrawData);
							encoderMode = select;
							break;

						case doExit:
						    goto end;
                    }
                }
    	}
    } /* while (1) */


    end:
    menudraw_deinit(MDrawData);

    textwindow_restoreSettings(Textwindowdump);

    return (ufast8_t) selectedItem;
}
