/** @file menu_color_gfx_elements.c
 *
 * Grafik-Elemente fuer Menues und andere Benutzerschnittstellen
 *
 * $Date: 2020-08-02 19:08:25 +0200 (So, 02 Aug 2020) $
 * $Revision: 32 $
 *
 * @date 01.11.2017 @author Nicolas */


#include "menu_color_gfx_elements.h"
#include "gfx_draw_lines.h"
#include "coord.h"
#include "gfx_draw.h"
#include "text_window.h"
#include "myassert.h"
#include <stdio.h>
#include "text_cursor.h"
#include "gfx_draw_rectangles.h"
#include "font.h"



/** Darstellung eines senkrechten Scrollbalkens (z.B. am rechten Bildrand)
 * @param[in] R: Position und Groesse des Scrollbalkens
 * @param[in] slPos: Aktuelle Position
 * @param[in] nSlPos: Groesse des Scrollbereichts (Anzahl der Positionen), nSlPos = 0 -> kein Slider
 * @param[in] C: Farbschema  */
void uiDraw_scrollbar(Rect_t R, ufast8_t slPos, ufast8_t nSlPos, const ScrollbarColor_t *C)
{
    R = gfx_cliprect(R);
    if(R.w < 1 || R.h < 1 )
    {   /* Scrollbar ausserhalb des Bildbereichs */
        return;
    }


    /* Rahmen */
    drawRect(R, (Colors_t) {C->frame, transparent});
    R = rect_clip(R, 1, align_all); /* Einen Schritt nach innen */
    drawRect(R, (Colors_t) {C->background, transparent});

    R = rect_clip(R, 1, align_left | align_right);
    R = rect_clip(R, 3, align_top  | align_bottom);

    /* Slider (wenn vorhanden) darstellen */
    if( nSlPos == 0 )
    {
        /* Slider voll ausgefuellt. */
        drawRectF(R, C->inactive);
    }
    else
    {
        if(slPos >= nSlPos ) slPos = nSlPos-1; /* Position begrenzen */

        /* Teilslider */
        uint_fast16_t hs = R.h/(nSlPos); /* Hoehe Slider-Segment */
        Rect_t Rsl = R;
        Rsl.h = slPos*hs;
        drawRectF(Rsl, C->background);
        Rsl.y += slPos*hs;
        Rsl.h = hs;
        drawRectF(Rsl, C->active);
        Rsl.y += hs;
        Rsl.h = (nSlPos-slPos-1)*hs;
        drawRectF(Rsl, C->background);
    }

	return;
}



/** Reine Dummy-Funktion */
void uiDraw_cursor(uint16_t cursorPos)
{
    (void) cursorPos;
}



/** Kopfzeilen o.Ae. darstellen (Text mit Icon links/rechts)
 * @param[in] R: Kopfzeilen-Position
 * @param[in] Cfg: Einstellungen
 * @param[in] C: Farbschema */
void uiDraw_headline(Rect_t R, const HeadlineConfig_t *const Cfg, const HeadlineColor_t *C)
{


    R = rect_clip(R, Cfg->margin, align_left | align_right);


    /* Text zentriert in Vordergrundfarbe ausgeben.
       Limitierung: Text muss in eine Zeile passen */
    if( Cfg->text != NULL )
    {

        Textwindowdump_t Dump = textwindow_dumpSettings();
        text_setWindow(R);
        textwindow_setColors((Colors_t) {.fg = C->text, .bg = C->background } );
        textfield(R, Cfg->text, align_center | align_middle);
        textwindow_restoreSettings(Dump);
    }


    /* Icons rechts und links */
    if( Cfg->iconLeft || Cfg->iconRight )
    {
        Colors_t Color = {C->icon, C->background};
        font_drawGlyphAligned(R, Cfg->iconFont, Cfg->iconLeft,  Color, align_left | align_middle);
        font_drawGlyphAligned(R, Cfg->iconFont, Cfg->iconRight, Color, align_right | align_middle);
    }
}
