/** @file menu_color_gfx_elements.h
 *
 * Grafik-Elemente fuer Menues und andere Benutzerschnittstellen
 *
 * $Date: 2020-08-01 20:41:52 +0200 (Sa, 01 Aug 2020) $
 * $Revision: 31 $
 *
 * @date 01.11.2017 @author Nicolas */


#ifndef GFX_COLOR_UI_H
    #define GFX_COLOR_UI_H

	#include <stdint.h>
	#include "color.h"
	#include "rect.h"


    /** Scrollbar: Farbschema-Einstellungen */
    typedef struct ScrollbarColor_s
    {
        color_t background;     /**< Hintergrundfarbe */
        color_t frame;          /**< Rahmen-Farbe */
        color_t active;         /**< Farbe aktiver Slider */
        color_t inactive;       /**< Farbe inaktiver Slider */
    }
    ScrollbarColor_t;



    /** Kopfzeile: Farbschema-Einstellungen */
    typedef struct HeadlineColor_s
    {
        color_t background;     /**< Hintergrundfarbe */
        color_t text;           /**< Testfarbe */
        color_t icon;           /**< Icon-Vordergrundfarbe */
    }
    HeadlineColor_t;



    /** Kopfzeile: Abmessungen und Einstellungen */
    typedef struct HeadlineConfig_s
    {
        const uint8_t *iconFont;/**< Schriftart Piktogramme */
        const char *text;       /**< Kopfzeilentext */
        uint8_t margin;         /**< Breite Randstreifen */
        uint8_t cornerRadius;   /**< Rundungsradius */
        uint16_t iconLeft;      /**< Piktogramm links (Unicode) */
        uint16_t iconRight;     /**< Piktogramm rechts (Unicode) */
    }
    HeadlineConfig_t;



	void uiDraw_scrollbar(Rect_t R, uint_fast8_t slPos, uint_fast8_t nSlPos, const ScrollbarColor_t *C);
    void uiDraw_headline(Rect_t R, const HeadlineConfig_t *const Cfg, const HeadlineColor_t *C);
    void uiDraw_cursor(uint16_t cursorPos);

#endif
