/** @file menu_color_private_color.h
 *
 * Menusystem (Hilfsfunktionen): Farben
 *
 * Zusammenstellung der Farben einzelner Elemente aus dem globalen Farb-Struct. Interne Funktionen.
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * 13.03.2018 Nicolas  */


#ifndef MENU_COLOR_PRIVATE_COLOR_H
    #define MENU_COLOR_PRIVATE_COLOR_H

    #include "menu_color.h"
    #include "menu_color_gfx_elements.h"


    /** Farbschema Scrollbar
     *
     * @param[in] Colorscheme: Menu-Farbschema
     * @return Scrollbar-Farbschema */
    static inline ScrollbarColor_t getScrollbarColor(const Menucolor_t Colorscheme)
    {
       ScrollbarColor_t CSb =
       {
            .background = Colorscheme.mainBackground,
            .frame      = Colorscheme.scrollbarFrame,
            .active     = Colorscheme.scrollbarActive,
            .inactive   = Colorscheme.scrollbarInactive,
        };
        return CSb;
    }



    /** Farbschema Kopfzeile
     *
     * @param[in] Colorscheme: Menu-Farbschema
     * @return Kopfzeilen-Farbschema */
    static inline HeadlineColor_t getHeadlineColor(const Menucolor_t Colorscheme)
    {
        HeadlineColor_t CHl =
        {
            .background = Colorscheme.headlineBackground,
            .text       = Colorscheme.headlineText,
            .icon       = Colorscheme.headlineIcon,
        };
        return CHl;
    }



    /** Farbschema Texthintergrund
     *
     * @param[in] Colorscheme: Menu-Farbschema
     * @return Text-Farbschema */
    static inline Colors_t getTextWindowColor(const Menucolor_t Colorscheme)
    {
        return (Colors_t) {Colorscheme.mainText, Colorscheme.mainBackground};
    }

#endif

