/** @file menu_color_private_control.h
 *
 * Menusystem (Hilfsfunktionen): Hilfsfunktionen operierend auf dem Menucontrol-Struct
 *
 * Menue: (Interne Funktionen.)
 *  - Drehgeber-Inkremente abfragen
 *  - Tastendruecke abfregen
 *  - externe Funktionen ausfuehren
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 16.03.2018 @author Nicolas  */


#ifndef MENU_COLOR_PRIVATE_CONTROL_H
    #define MENU_COLOR_PRIVATE_CONTROL_H

    #include "menu_color.h"


    /** Hilfsfunktion: Action-Funktion ausfuehren
     * (Tastendruck, Enter o.Ae. abfragen)
     *
     * @param[in] Control: Menue-Control-Struct
     * @return Rueckgabewert der Action-Funktion */
    static bool menucontrol_getAction(Menucontrol_t const* Control)
    {
        bool (*fcn)(void) = Control->pActnFcn;
        return ( fcn==NULL ) ? 0 : (fcn)();
    }



    /** Hilfsfunktion: Inkrement-Funktion ausfuehren
     * (Encoder-Drehung, Pfeiltasten o.Ae. abfragen)
     *
     * @param[in] Control: Menue-Control-Struct
     * @return Rueckgabewert der Inkrmenent-Funktion */
    static int menucontrol_getIncrement(Menucontrol_t const* Control)
    {
        int (*fcn)(void) = Control->pIncrFcn;
        return ( fcn==NULL ) ? 0 : (fcn)();
    }



    /** Hilfsfunktion: Action- und Inkrement-Funktion ruecksetzen
     *
     * Dreheber und evtl. anstehende Tastendruecke werden zurueckgesetzt, indem ihre Funktion
     * jeweils zweimal ausgefuehrt wird.
     *
     * @param[in] Control: Menue-Control-Struct
     * @return Rueckgabewert der Action-Funktion */
    static void menucontrol_reset(Menucontrol_t const* Control)
    {
        menucontrol_getIncrement(Control);
        menucontrol_getIncrement(Control);
        menucontrol_getAction(Control);
        menucontrol_getAction(Control);
    }



    /** Hilfsfunktion: Action- und Inkrement-Funktion "pruefen" und ruecksetzen
     *
     * @param[in] Control: Menue-Control-Struct
     * @return Rueckgabewert der Action-Funktion */
    static void menucontrol_init(Menucontrol_t const* Control)
    {
        assert( Control != NULL );
        assert( Control->pActnFcn != NULL );
        assert( Control->pIncrFcn != NULL );

        menucontrol_reset(Control);
    }



    /** Hilsfunktion: Init-Funktion ausfuehren
     *
     * @param[in] Control: Menue-Control-Struct */
    static void menucontrol_executeExternalInitFcn(Menucontrol_t const* Control)
    {
        void (*fcn)(void) = Control->pInitFcn;
        if( fcn!=NULL )
        {
            (fcn)();
        }
    }



    /** Hilsfunktion: Schleifen-Funktion ausfuehren
     *
     * @param[in] Control: Menue-Control-Struct */
    static void menucontrol_executeExternalLoopFcn(Menucontrol_t const* Control)
    {
        void (*fcn)(void) = Control->pRptFcn;
        if( fcn!=NULL )
        {
            (fcn)();
        }
    }

#endif

