/** @file menu_color_private_geometry.h
 *
 * Menuesystem (Hifsfunktionen): Berechnungen zur Geometrie zur Menudarstellung
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 13.03.2018 @author Nicolas  */


#ifndef MENU_COLOR_PRIVATE_GEOMETRY_H
    #define MENU_COLOR_PRIVATE_GEOMETRY_H

    #include "menu_color.h"


    /** Kopfzeile: Position und Groesse
     *
     * @param[in] Dim: Menue-Groessen-Einstellungs-Struct
     * @return Rechteck Kopfzeile */
    static inline Rect_t menu_private_getHeadBackgroundArea(const Menudimension_t Dim)
    {
        Rect_t Hl = Dim.Area;
        Hl.h = Dim.headheight;
        return Hl;
    }



    /** Kopfzeile: Position und Groesse des Textfeldes
     *
     * @param[in] Dim: Menue-Groessen-Einstellungs-Struct
     * @return Rechteck Kopfzeilen-Textfeld */
    static inline Rect_t menu_private_getHeadTextArea(const Menudimension_t Dim)
    {
        Rect_t Hl = Dim.Area;
        Hl.h = Dim.headheight;
        if( Dim.headsep < 0 )
            Hl.h += Dim.headsep;
        return Hl;
    }



    /** Menu-Tabellenkoerper: Position und Groesse
     *
     * @param[in] Dim: Menue-Groessen-Einstellungs-Struct
     * @return Rechteck Tabellenkoerper */
    static inline Rect_t menu_private_getBodyArea(const Menudimension_t Dim)
    {
        Rect_t Bg = Dim.Area;
        ifast16_t topMargin = Dim.headheight + Dim.headsep;
        Bg.y += topMargin;
        Bg.h -= topMargin;
        return Bg;
    }



    /** Menu-Tabellenkoerper: Position und Groesse des Textfeldes
     *
     * @param[in] Dim: Menue-Groessen-Einstellungs-Struct
     * @return Rechteck Menu-Textfeld */
    static inline Rect_t menu_private_getTextArea(const Menudimension_t Dim)
    {
        Rect_t Bg = Dim.Area;

        ifast16_t extraTopBottomMargin = 2;
        ifast16_t topMargin = Dim.headheight + Dim.headsep + extraTopBottomMargin;
        ifast16_t leftMargin = Dim.margin + Dim.textIndent;
        ifast16_t bottomMargin = extraTopBottomMargin;

        /* Rechter Rand: Scrollbalken + Rand rechts und links vom Scrollbalken */
        ifast16_t rightMargin = Dim.scrollbarWidth + 2*Dim.margin;

        Bg.y += topMargin;
        Bg.h -= topMargin + bottomMargin;
        Bg.x += leftMargin;
        Bg.w -= leftMargin + rightMargin;
        return Bg;
    }



    /** Menu-Scroll: Position und Groesse
     *
     * @param[in] Dim: Menue-Groessen-Einstellungs-Struct
     * @return Rechteck Scrollbar */
    static inline Rect_t menu_private_getScrollbarArea(const Menudimension_t Dim)
    {
        Rect_t Sb = Dim.Area;
        ifast16_t topMargin = Dim.headheight + Dim.headsep;
        ifast16_t leftMargin = Sb.w - Dim.scrollbarWidth - Dim.margin;
        Sb.x += leftMargin;
        Sb.w =  Dim.scrollbarWidth;
        Sb.y += topMargin + Dim.scrollbarWidth/2;
        Sb.h -= topMargin + Dim.scrollbarWidth;
        return Sb;
    }

#endif
