/** @file menue_templates_examples.c
 *
 * Copy-Paste-Vorlagen fuer unterschiedliche Menues und ihre Darstellung
 *
 *  * Farbwahl
 *  * Steuerung
 *  * Startfunktion
 *
 * $Date: 2020-08-13 11:28:54 +0200 (Do, 13 Aug 2020) $
 * $Revision: 68 $
 *
 * @date 12.03.2018 @author Nicolas */


#include "menue_templates.h"
#include "sw_config.h"
#include "locale.h"
#include "menu_color.h"
#include "gfx_draw.h"
#include "pgmspace.h"
#include "myassert.h"
#include <stddef.h>
#include "u8g2_fontdata.h"
#include "userinterface.h"


#if defined( DOXYGEN )

    /** Hilfsfunktion: Hintergrund ausblenden
     *
     * Der aktuelle Bildschirminhalt wird grau mit 50% Deckungsgrad uebermalt. Das eignet sich als
     * Startfunktion fuer Menues aller Art. */
    void overlay(void)
    {
        #if defined( INCLUDE_TFT_FUNCTIONS )
            Rect_t R = gfx_getDisplayDimensions();
            tft_drawRectTransparent(R, gray, 64);
        #endif
    }



    /** Hilfsfunktion: Aktion ausloesen: Generische Aktions-Funktion in booleschen Wert umwandeln */
    bool getAction(void)
    {
        return !!ui_getAction();
    }



    /** Menue-Farbschema: Dunkel */
    constflash Menucolor_t ColorScheme_dark =
    {
        .mainBackground = black,
        .mainText = white,
        .cursorText = white,
        .cursorBackground = blue,
        .cursorFrame = black,
        .scrollbarActive = white,
        .scrollbarInactive = gray,
        .scrollbarFrame = red,
        .headlineBackground = black,
        .headlineText = yellow,
        .headlineIcon = red,
    };



    /** Menue-Farbschema: Hell */
    constflash Menucolor_t ColorScheme_light =
    {
        .mainBackground = white,
        .mainText = blue,
        .cursorText = white,
        .cursorBackground = blue,
        .cursorFrame = black,
        .scrollbarActive = black,
        .scrollbarInactive = gray,
        .scrollbarFrame = blue,
        .headlineBackground = white,
        .headlineText = black,
        .headlineIcon = green,
    };



    /** Menu-Farbschema: Schrittmotorpruefstand
     *
     * Gesondertes Farbschema fuer den Schrittmotorpruefstandsmodus:
     * hellblauer Hintergrund mit rosafarbenen Elementen - kitschig und auffaellig */
    constflash Menucolor_t ColorScheme_testbench =
    {
        .mainBackground     = 0x8D1C,
        .mainText = white,
        .cursorText = white,
        .cursorBackground = 0x8D1C,
        .cursorFrame = black,
        .scrollbarActive = white,
        .scrollbarInactive = gray,
        .scrollbarFrame = red,
        .headlineBackground = 0xE571,
        .headlineText = white,
        .headlineIcon = white,
    };



    /** Menu-Farbschema: S/W-LCD
     *
     * Schwarze Kopfzeile, Piktrogramme weiss auf schwarzem Grund, Menuelement schwarz auf weiss */
    constflash Menucolor_t ColorScheme_Monochrome =
    {
        .mainBackground     = 0,
        .mainText = 1,
        .cursorText = 0,
        .cursorBackground = 1,
        .cursorFrame = transparent,
        .scrollbarActive = white,
        .scrollbarInactive = gray,
        .scrollbarFrame = red,
        .headlineBackground = 1,
        .headlineText = 0,
        .headlineIcon = 0,
    };



    /** Menu-Abmessungen: Standardmenue
     *
     * Ein TFT mit den Abmessungen 320 x 240 Pixel wird gut ausgenutzt. Die Kopfzeile bietet Platz
     * fuer zwei Piktogramme 32x32 Pixel und eine Zeile Text. Am Rand scheint noch der vorherige
     * Hintergrund sichtbar. */
    constflash Menudimension_t Menudimension_standard =
    {
        .Area = {20, 5, 280, 230},
        .headheight = 40,
        .scrollbarWidth = 10,
        .margin = 4,
        .textIndent = 4,
        .cornerRadius = 5,
        .headsep = 4,
        .textcellwidth = {9, 0, 5, 4},
        .textalign = {align_left, align_center, align_right, align_left},
    };



    /** Menue-Einstellungen: Standardmenue
     *
     *  * Zum Start wird der Hintergrund grau ausgeblendet
     *  * Der Drehgeber steuert Auswahl und Inkrement/Dekrement
     *  * Keine Schleifenfunktion */
    constflash Menucontrol_t Menucontrol_standard =
    {
        .pIncrFcn = ui_getIncrementFast,
        .pActnFcn = getAction,
        .pInitFcn = overlay,
        .pRptFcn  = NULL,
    };



    /** Einstellungen: Einfaches Standard-Menue */
    constflash Menuconfig_t MenuConfig_Main =
    {
        .Dimension = &Menudimension_standard,
        .Color = &ColorScheme_dark,
        .Control   = &Menucontrol_standard,
        .headtext = NULL,
        .iconFont = open_iconic_selection_4x,
    };

#endif
