/** @file messagebox.c
 *
 * Darstellung von Popup-/Dialog-Fenstern
 *
 * Dialogfenster sind Menuefenster mit vereinfachter Konfiguration.
 *
 * $Date: 2020-08-12 13:24:37 +0200 (Mi, 12 Aug 2020) $
 * $Revision: 61 $
 *
 * @date 10.07.2013 @author Nicolas  */


#include "messagebox.h"
#include "sw_config.h"
#include "gfx_draw.h"
#include "menue_templates.h"
#include "locale.h"
#include "u8g2_fontdata.h"
#include "u8g2_fontdata_icons.h"
#include <string.h>
#include <stdio.h>



/* Menueeinstellungen extern definiert, z.B. in Projekt-Datei "menue_templates.c" */
extern constflash Usermessage_Type_t Usermessage_Types[];
extern constflash Menuconfig_t MenuConfig_Messagebox_default;


#if defined( DOXYGEN )
    /* Beispielkonfiguration fuer TFT-Farbdisplay */

    /** Messagebox: Piktogramme zusammengestellt fuer Font open_iconic_selection_4x */
    constflash Usermessage_Type_t Usermessage_Types[] =
    {
        [messagetype_info] = {.icon=icon_info_x, .iconColor=blue },
        [messagetype_error] = {.icon=icon_circle_x, .iconColor=red},
        [messagetype_success] = {.icon=icon_circle_check, .iconColor=green},
        [messagetype_end] = {.icon=0, .iconColor=yellow},
    };


    /** Messagebox: Default-Einstellungen
     * Piktogramme und Kopfzeilentext werden zur Laufzeit ausgetauscht. */
    constflash Menuconfig_t MenuConfig_Messagebox_default =
    {
        .Dimension  = &Menudimension_messagebox,
        .Color      = &ColorScheme_light,
        .Control    = &Menucontrol_standard,
        .headtext   = buffer,
        .iconLeft   = icon,
        .iconRight  = icon,
        .iconFont   = open_iconic_selection_4x,
    };

#endif




/** Textbox/Popup/Auswahlmenue
 *
 * Menue-Fenster mit stark eingeschraenkten Konfigurationsmoeglichkeiten fuer die Verwendung als
 * einfacher Benutzer-Dialog.
 *
 * Benoetigt 100 Bytes fuer einen Textpuffer und Platz fuer einen Satz Menue-Konfigurationsdaten.
 *
 * @param[in] Message: Konfigurationsdaten der darzustellenden Nachricht (inkl. Auswahlmenue)
 * @return Index des ausgewahltes Elements */
ufast8_t messagebox(Usermessage_t *Message)
{
    /* Menueeintraege */
    ufast8_t nItem = 0;
    while(strlen_P(Message->items[nItem]) ) nItem++;

    /* Items zusammenstellen */
    Menuitem_t MenuData[nItem+1];
    MenuData[nItem] = (Menuitem_t) { "", .All = {endmarker} };
    for( uint8_t i = 0; i< nItem; i++ )
    {
        MenuData[i] = (Menuitem_t) { Message->items[i] , .All     ={ exitMenu }        };
    }

    /* Kopfzeile interpretieren (String + Int) */
    char buffer[100];
    snprintf(buffer, 99, Message->text, Message->errcode);


    /* Nur gueltige Werte fuer Messagetype erlauben */
    uint_fast8_t messagetype = MIN(Message->type, messagetype_end);

    /* Piktogramme und Farbschema auswahlen */
    uint16_t icon = Usermessage_Types[messagetype].icon;
    color_t iconColor = Usermessage_Types[messagetype].iconColor;

    /* Lokale Kopien der Menueeinstellungen erzeugen, um sie aendern zu konnen */
    Menucolor_t Color = *(MenuConfig_Messagebox_default.Color);
    Color.headlineIcon = iconColor;

    Menuconfig_t MenuConfig = MenuConfig_Messagebox_default;
    MenuConfig.iconLeft  = icon;
    MenuConfig.iconRight = icon;
    MenuConfig.headtext = buffer;
    MenuConfig.Color = &Color;


    return menu_standard(MenuData, &MenuConfig, 0);
}
