/** @file messagebox.h
 *
 * Darstellung von Popup-/Dialog-Fenstern
 *
 * Dialogfenster sind Menuefenster mit vereinfachter Konfiguration.
 *
 * $Date: 2020-08-03 18:14:55 +0200 (Mo, 03 Aug 2020) $
 * $Revision: 33 $
 *
 * @date 10.07.2013 @author Nicolas  */


#ifndef MESSAGEBOX_H
    #define MESSAGEBOX_H

    #include "color.h"
    #include "pgmspace.h"
    #include "menu_color.h"
    #include "nstdint.h"

    typedef constflash char * const usermessageitem_t;


    /** Nachrichten-Typ (bestimmt Farben und Piktogramm) */
    enum messagebox_messagetype_e
    {
        messagetype_info = 0,   /**< informativ -> neutrale Farbe und Icon */
        messagetype_error,      /**< Fehler -> rote Farbe, Fehler-Icon */
        messagetype_success,    /**< Erfolg -> gruene Farbe, Erfolgs-Icon */
        messagetype_end,        /**< Nach dem letzten gueltigen Item */
    };


    /** Message-Box: Benutzernachricht */
    typedef struct Usermessage_s
    {
        enum messagebox_messagetype_e type; /**< Art der Nachricht */
        constflash char *text;              /**< Text (wird von "printf" interpretiert), eine
                                              einzelne int-Variable erlaubt, typischerweise
                                              ein Fehlercode */
        int errcode;                        /**< Der Fehlercode (als Int-Variable des printf() */
        constflash char * const *items;     /**< Auswahlemente (String-Array) */
        uint8_t headheight;                 /**< Hoehe der Kopfzeile (fuer Text-String) */
    }
    Usermessage_t;



    /** Message-Box: Einstellungen Message-Typen */
    typedef struct Usermessage_Type_s
    {
        uint16_t icon;          /**< Unicode des Piktogramms */
        color_t iconColor;      /**< Vordergrundfarbe des Piktogramms */
    }
    Usermessage_Type_t;



    ufast8_t messagebox(Usermessage_t *message);


#endif

