/*
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 */




#include "wallclock.h"
#include <stdint.h>
#include "unity_fixture.h"
#include "delay.h"

TEST_GROUP(wallclock);
TEST_SETUP(wallclock) {}
TEST_TEAR_DOWN(wallclock) {}

TEST_GROUP_RUNNER(wallclock)
{
    RUN_TEST_CASE(wallclock, tictoc);
    RUN_TEST_CASE(wallclock, ms2wallclocktime);
    RUN_TEST_CASE(wallclock, timeout);
}





TEST(wallclock, tictoc)
{
    wallclock_t a;

    #if defined( MOCKUP_SDL ) || defined( STM32F4XX )|| defined( STM32F10x )

        // Auf dem echten Target/Mockup laeuft der Timer im Hintergrund weiter - Test
        // ergibt also nicht viel Sinn.
        TEST_ASSERT_NOT_EQUAL( 0, a = wallclock_tic() );
        delay_ms(200);
        TEST_ASSERT_NOT_EQUAL( 0, wallclock_toc_ms(a) );


    #else
        TEST_ASSERT_EQUAL( 0, a = wallclock_tic() );
        TEST_ASSERT_EQUAL( 0, wallclock_toc_ms(a) );

        TEST_ASSERT_EQUAL( 0, a = wallclock_tic() );
        counter_milliseconds++;
        TEST_ASSERT_EQUAL( 1, wallclock_toc_ms(a) );
    #endif

    // Ueberlauf testen
    counter_milliseconds = UINT32_MAX - 3;
    TEST_ASSERT_EQUAL( UINT32_MAX-3, a = wallclock_tic() );
    counter_milliseconds += 6;
    TEST_ASSERT_EQUAL( 6, wallclock_toc_ms(a) );

    // Testen, dass ein Timer auch als Abbruchbedingung taugt
    a = wallclock_tic();
    while( wallclock_toc_ms(a) < 1000 ) counter_milliseconds++;
    TEST_ASSERT_MESSAGE( 1, "no break");
}


TEST(wallclock, ms2wallclocktime)
{
    Wallclock_time_t time = wallclock_ms2wallclocktime(1);
    TEST_ASSERT_EQUAL( 1, time.ms );
    TEST_ASSERT_EQUAL( 0, time.s );

    time = wallclock_ms2wallclocktime(1001);
    TEST_ASSERT_EQUAL( 1, time.ms );
    TEST_ASSERT_EQUAL( 1, time.s );

    // 2^32-1 ms sind 49 d, 17 h, 2 min, 47 s und 295 ms
    time = wallclock_ms2wallclocktime(UINT32_MAX);
    TEST_ASSERT_EQUAL( 49, time.d );
    TEST_ASSERT_EQUAL( 295, time.ms );
    TEST_ASSERT_EQUAL( 47, time.s );
    TEST_ASSERT_EQUAL( 2, time.m );
    TEST_ASSERT_EQUAL( 17, time.h );
}



TEST(wallclock, timeout)
{
    wallclock_t timeout = wallclock_setTimeout(10);
    TEST_ASSERT_EQUAL(0, wallclock_checkTimeout(timeout));
    counter_milliseconds += 10;
    TEST_ASSERT_EQUAL(1, wallclock_checkTimeout(timeout));

    counter_milliseconds = UINT32_MAX - 3;
    timeout = wallclock_setTimeout(10);
    TEST_ASSERT_EQUAL(0, wallclock_checkTimeout(timeout));
    counter_milliseconds += 9;
    TEST_ASSERT_EQUAL(0, wallclock_checkTimeout(timeout));
    counter_milliseconds += 10;
    TEST_ASSERT_EQUAL(1, wallclock_checkTimeout(timeout));

    counter_milliseconds = UINT32_MAX - 13;
    timeout = wallclock_setTimeout(10);
    TEST_ASSERT_EQUAL(0, wallclock_checkTimeout(timeout));
    counter_milliseconds += 10;
    TEST_ASSERT_EQUAL(1, wallclock_checkTimeout(timeout));
    counter_milliseconds += 10;
    TEST_ASSERT_EQUAL(1, wallclock_checkTimeout(timeout));
}

