/** @file mocksdl_display.c
 *
 * Einfache Emulation des Displayverhaltens ueber SDL2
 *
 * Die Emulation beschraenkt sich darauf, den Displaypuffer grafiscsh
 * darzustellen. Das Timing und die tatsaechliche Kommunikation mit dem GLCD
 * wird nicht beruecksichtigt. Ausserdem funktioniert nur der Modus mit
 * DISPLAYBUFFER.
 *
 * $Date: 2020-08-12 16:39:52 +0200 (Mi, 12 Aug 2020) $
 * $Revision: 66 $
 *
 * @date 19.04.2015 @author Nicolas */


#include "mocksdl_display.h"
#include <stdio.h>
#include "sw_config.h"



#if defined( INCLUDE_MOCKUP_SDL )

    #include "hw_config.h"
    #include "gfx_draw.h"
    #include <SDL2/SDL.h>
    #include "mocksdl.h"




    /* Skalierfaktor */
    #if defined( GLCD_EMULATION_SCALING )
        #define DISPLAY_EMULATION_SCALING  GLCD_EMULATION_SCALING

    #elif defined( TFT_EMULATION_SCALING )
        #define DISPLAY_EMULATION_SCALING  TFT_EMULATION_SCALING

    #elif defined( DISPLAY_EMULATION_SCALING )

    #else
        #error "Display emulation scaling is not defined "

    #endif



    /* Variablen fuer Display-Emulation */
    SDL_Window   * pWindow;
    SDL_Renderer * pRenderer;
    SDL_Texture * pTexture;



    /** Grafik-Teil des Mockups mit SDL initialisieren
     *
     * @return EXIT_SUCCESS */
    exit_t mocksdl_display_init(void)
    {
        sdl_init();

        int ret = SDL_InitSubSystem(SDL_INIT_VIDEO);
        assert( ret == 0 );

        int width_px = gfx_getDisplayDimensions().w;
        int height_px = gfx_getDisplayDimensions().h;

        char headline[80];
        snprintf(headline, 79, "Display Emulation %i x %i", width_px, height_px);

        pWindow = SDL_CreateWindow(headline,
                SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED,
                DISPLAY_EMULATION_SCALING*width_px,
                DISPLAY_EMULATION_SCALING*height_px,
                0);

        pRenderer = SDL_CreateRenderer(pWindow, -1, 0);
        SDL_RenderSetScale(pRenderer, 1.0 * DISPLAY_EMULATION_SCALING, 1.0 * DISPLAY_EMULATION_SCALING);

        uint32_t pixelformat = SDL_PIXELFORMAT_RGB565;

        pTexture = SDL_CreateTexture(pRenderer, pixelformat,
                                     SDL_TEXTUREACCESS_STATIC, width_px, width_px);

        return EXIT_SUCCESS;
    }



    /** Vorschriftsmaessige Beendigung vorbereiten */
    void mocksdl_display_deinit(void)
    {
        SDL_DestroyTexture(pTexture);
        SDL_DestroyRenderer(pRenderer);
        SDL_DestroyWindow(pWindow);
        SDL_QuitSubSystem(SDL_INIT_VIDEO);
        printf("SDL deinit()\n");
    }


#else
    typedef int this_is_no_empty_translation_unit;
#endif





