/** @file mocksdl_timer.c
 *
 * Einfache Emulation von Timern ueber SDL2
 *
 * Der Timer kann primaer dazu verwendet werden, Teile der Hardware zu
 * simulieren oder emulieren, ohne dass der Kontrollfluss angepasst werden muss.
 *
 * $Date: 2020-08-12 16:39:52 +0200 (Mi, 12 Aug 2020) $
 * $Revision: 66 $
 *
 * @date 20.04.2015 @author Nicolas
 */



#include "mocksdl_timer.h"
#include "sw_config.h"


#if defined( INCLUDE_MOCKUP_SDL )

    #include "mocksdl.h"
    #include <stdint.h>
    #include <stdio.h>
    #include <stdbool.h>
    #include <SDL2/SDL.h>
    #include "assert.h"




    extern void SysTick_Handler(void);

    #define SDL_TIMER_INTERVAL_MS (SYSTICK_TIMER_INTERVAL_MS)



    SDL_TimerID timer_id;


    /* Emulation eines IRQ-aehnlichen Verhaltens, indem die Timer-Funktion von Hand
     * aufgerufen wird */
    static uint32_t mock_timer_callback(uint32_t interval, void *param)
    {
        (void) param;
        SysTick_Handler();
        return interval;
    }




    /* Timer mit Callback-Funktion einrichten */
    void mocksdl_timer_init(void)
    {
        /// @fixme Nanu?
        //void sdl_init(void);
        sdl_init();

        int ret = SDL_InitSubSystem(SDL_INIT_TIMER);
        assert( ret == 0 );

        uint32_t delay = SDL_TIMER_INTERVAL_MS;  /* To round it down to the nearest 10 ms */
        SDL_TimerID my_timer_id = SDL_AddTimer(delay, mock_timer_callback, NULL);

        timer_id =  my_timer_id;
    }


    void mocksdl_timer_deinit(void)
    {
        SDL_RemoveTimer(timer_id);
        SDL_QuitSubSystem(SDL_INIT_TIMER);
    }


#else
    typedef int this_is_no_empty_translation_unit;
#endif
