/** @file myassert.h
 *
 * Re-Implementierung des assert-Standard-Headers, um targetspezifische Eigenheiten zu
 * beruecksichtiten, insbesondere der Notwendigkeit eines manuellen Refreshs der Ausgabe.
 *
 * $Date: 2020-08-13 11:28:54 +0200 (Do, 13 Aug 2020) $
 * $Revision: 68 $
 *
 * @date 30.04.2014 uthor Nicolas */


#ifndef MYASSERT_H
    #define MYASSERT_H

    #undef assert
    #undef error

	extern void abort(void);

    #if defined(NDEBUG)
		#define assert(e) ((void) 0)

    #else
		/* Es wird nur der Funktions- und nicht der Dateiname ausgegeben, da
		 * Dateinamen mit dem vollen Pfad zuviel Display-Flaeche verdecken */

	    #include "pgmspace.h"
        #include <stdbool.h>


        /* Fuer Modultests */
        void prepareErrorCondition(void);
        bool isErrorConditionTriggered(void);
        void clearErrorCondition(void);


        void triggerHardFault(void);


        /** Assertion
         *
         * Wenn Assertion nicht erfuellt, Fehlermeldung ausgeben und abort()
         * Aufruf: assert(a<0);
         *
         * @param[in] e: Bedingung */
        #define assert(e) ((e) ? (1) : \
                assert_P_dontCallMe(__func__, __LINE__, PSTR(#e)) )



        /** Fehler
         *
         * Fehlermeldung ausgeben und abort()
         * Aufruf: error("Es ist so schrecklich!");
         *
         * @param[in] e: Fehlermeldung */
        #define error(e) error_P_dontCallMe(__func__, __LINE__, PSTR(e))


		/* assert_P_dontCallMe und error_P_dontCallMe sollen nicht direkt
         * aufgerufen werden! */
        int assert_P_dontCallMe(const char *f, const int l, constflash char *e);
        void error_P_dontCallMe(const char *f, const int l, constflash char *e);

	#endif

    #ifndef static_assert
        #define static_assert __extension__ _Static_assert
    #endif


#endif /* ASSERT_H */
