/** @file mystring.c
 *
 * Ergaenzungen zu den Funktionen der <string.h>
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 18.03.2018 @author Nicolas */

#include "mystring.h"
#include <stdlib.h>



#if defined( __MINGW64__ ) || defined( DOXYGEN )

    /** strsep(): strsep - extract token from string (Eigen-Implementierung)
     *
     * Nicht alle Varianten der Libc besitzen strsep().
     * Quelltext aus den Weiten des Internets http://unixpapa.com/incnote/string.html
     * Beschreibung aus den Linux-Manpages
     *
     * Be cautious when using this function. If you do use it, note that:
     *
     * * This function modifies its first argument.
     *
     * * This function cannot be used on constant strings.
     *
     * * The identity of the delimiting character is lost.
     *
     * If param stringp is NULL, the strsep() function returns NULL and does nothing else. Otherwise,
     * this function finds the first token in the string *stringp, where tokens are delimited by
     * symbols in the string delim. This token is terminated by overwriting the delimiter with a
     * null byte ('\0') and *stringp is updated to point past the token. In case no delimiter was
     * found, the token is taken to be the entire string *stringp, and *stringp is made NULL.
     *
     * @param[in] stringp
     * @param[in] delim
     * @return a pointer to the token, that is, it returns the original value of *stringp */
    char * strsep(char **stringp, char *delim)
    {
        char *p, *s;
        if (stringp == NULL || *stringp == NULL || **stringp == '\0') return(NULL);
        s = *stringp;
        p = s + strcspn(s, delim);
        if (*p != '\0') *p++ = '\0';
        *stringp = p;
        return(s);
    }
#endif


