/** @file nstdint.h
 *
 * Nicht-Standard-Integer-Datentypen
 *
 * @date 28.11.2017 @author Nicolas */


#ifndef NSTDINT_H_INCLUDED
    #define NSTDINT_H_INCLUDED

    #include <stdint.h>

    /* Plattformspezifische Datentypen */
    #ifdef AVR
        typedef __int24  int_least24_t;
        typedef __uint24 uint_least24_t;
        typedef __int24  int_fast24_t;
        typedef __uint24 uint_fast24_t;
        typedef __uint24 uint24_t;

    #elif defined(STM32F10x) || defined(STM32F4XX)

        typedef int32_t  int_least24_t;
        typedef uint32_t uint_least24_t;
        typedef int32_t  int_fast24_t;
        typedef uint32_t uint_fast24_t;

        #define UINT_FAST24_MAX UINT32_MAX

     #elif defined _MSC_VER
        typedef int32_t  int_least24_t;
        typedef uint32_t uint_least24_t;
        typedef int32_t  int_fast24_t;
        typedef uint32_t uint_fast24_t;


    #elif defined( __MINGW64__ ) || defined( __MINGW32__ ) || defined( __linux__ )
        #include <stdlib.h>

        /** 24-Bit-Datentypen:
           MinGW kann keinen genau 24 Bit breiten Datentyp zur Verfuegung stellen.
           uint24_t muss also situationsabhaengig definiert/vermieden werden. */
        typedef int32_t  int_least24_t;
        typedef uint32_t uint_least24_t;
        typedef int32_t  int_fast24_t;
        typedef uint32_t uint_fast24_t;

    #else
        #error "No MCU defined"

    #endif

	/* Plattformunabhaengige Datentypen */
    typedef int32_t fix16_16_t;

    /* Kurznamen fuer Datentypen aus <stdint.h> */
    typedef uint_fast8_t ufast8_t;
    typedef uint_fast16_t ufast16_t;
    typedef uint_fast24_t ufast24_t;
    typedef int_fast8_t ifast8_t;
    typedef int_fast16_t ifast16_t;
    typedef int_fast24_t ifast24_t;



#endif // DATATYPES_H
