/* syscalls.c
 *
 * Versuch des retargetings fuer printf etc.
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * 26.02.2018 Nicolas */


#include <stddef.h>

#if defined(STM32F10x) || defined(STM32F4XX)
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <stdint.h>
#undef errno

#include <reent.h>
#include "tft_text.h"


// https://www.mikrocontroller.net/articles/STM32_Eclipse_JLink_Linux/Windows#Optional:_Syscalls_implementieren

// Add the #include directive for the used micro controller here.

#if defined (STM32F40_41xxx) || defined (STM32F427_437xx) || defined (STM32F429_439xx) || defined (STM32F401xx) || defined (STM32F411xE)
	#include <stm32f4xx.h>
	#error "Das hier wird gar nicht erreicht"
#endif
#if defined(STM32F37X)
	#include <stm32f37x.h>
#endif
#if defined(STM32F302x8)
	#include <stm32f30x.h>
#endif

// This macro can be used to add function attributes to all  functions in this file if neccessary.
#define SYSCALL_FUN_ATTR

// Use this definition if you encounter linker errors like: `_fstat_r' referenced in section ... defined in discarded section `.text' of ./syscalls.o (symbol from plugin) when compiling with LTO
//#define SYSCALL_FUN_ATTR __attribute__ ((used))


ssize_t _write_r(struct _reent * reent, int fd, const void *buf, size_t cnt) SYSCALL_FUN_ATTR;
void _exit(int status) SYSCALL_FUN_ATTR;
void *_sbrk_r(struct _reent * reent, ptrdiff_t incr) SYSCALL_FUN_ATTR;
int _close_r(struct _reent * reent, int fd) SYSCALL_FUN_ATTR;
int _fstat_r(struct _reent * reent, int fd, struct stat *pstat) SYSCALL_FUN_ATTR;
int _isatty_r(struct _reent * reent, int file) SYSCALL_FUN_ATTR;
off_t _lseek_r(struct _reent * reent, int fd, off_t pos, int whence) SYSCALL_FUN_ATTR;
ssize_t _read_r(struct _reent * reent, int fd, void *buf, size_t cnt) SYSCALL_FUN_ATTR;
int _kill(int pid, int sig) SYSCALL_FUN_ATTR;
int _getpid(void) SYSCALL_FUN_ATTR;


// Prints string to stdout
 __attribute__((used)) ssize_t _write_r(struct _reent * reent, int fd, const void *buf, size_t cnt)
{
	if (fd != 1 && fd != 2)
    {
		((struct _reent*) reent)->_errno = EBADF;
		return -1;
	}

    #if 1
	// Prints the string to SWO output. May be change to e.g. UART output.
        tft_putstrn(buf, cnt);
        return cnt;
	#else
        // Prints the string to SWO output. May be change to e.g. UART output.
        if ((ITM->TCR & 1) == 0)
            return cnt;
        if ((ITM->TER & 1) == 0)
            return cnt;

        for (size_t i = 0; i < cnt; ++i) {
            while ((ITM->PORT[0].u8 & 1) == 0);
            ITM->PORT[0].u8 = ((uint8_t*) (buf))[i];
        }
        return cnt;
	#endif
}


#endif
