/** @file nvram.h
 *
 * Nichtfluechtiger Speicher
 *
 * Behandlung des nichtfluechtigen Speichers, insbesondere Lesen und Speichern
 * von Konfigurationsdaten. Es werden Pruefsummen gebildet und ueberprueft.
 * Das bedingt einen Overhead von 10 Bytes je Datensatz - bei der Bildung
 * von Adressen ist das zu beruecksichtigen.
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * @date 04.05.2015 @author Nicolas */


#ifndef NVRAM_H
    #define NVRAM_H

    #include <stdint.h>



	enum nvram_status_e
    {
		nvram_ok = 0,
		nvram_unknown_failure = 1,
		nvram_i2c_failure = 2,
		nvram_magic_failure = 3,
		nvram_crc_failure = 4
	};


	uint_fast8_t nvram_read(uint8_t *buffer, uint_fast8_t address, uint_fast8_t size);
	uint_fast8_t nvram_write(uint8_t *dataset, uint_fast8_t address, uint_fast8_t size);

#endif
