/*
 * $ Date:  $
 * $ Revision: $
 */



#include "sw_config.h"



#if defined( INCLUDE_UNIT_TESTS )

    #include "text_field.h"
    #include "unity_fixture.h"
//    #include "tft.h"
    #include <string.h>
    #include "myassert.h"
    #include "sw_config.h"
    #include "gfx_draw.h"
    #include "font.h"
    #include "u8g2_fontdata.h"
    #include "text_cursor.h"

    Textwindowdump_t Dump;

    TEST_GROUP(text_field);
    TEST_SETUP(text_field)
    {
        // Textfont sichern, damit bei Modultests auf der Zielplattform nicht die komplette
        // Ausgabe geschrottet wird.
        Dump =  textwindow_dumpSettings();
        textwindow_setFont(font_largestFontAvailable);
    }


    TEST_TEAR_DOWN(text_field)
    {
        textwindow_restoreSettings(Dump);
    }

    TEST_GROUP_RUNNER(text_field)
    {
        RUN_TEST_CASE(text_field, text_field);
        RUN_TEST_CASE(text_field, formatString2array);
        RUN_TEST_CASE(text_field, uarray16_stretchDistance);
    }


    TEST(text_field, text_field)
    {

        Coord2_t C = cursor_getPosition();
        Rect_t R = gfx_getDisplayDimensions();
        Rect_t BB;
        char s[] = "lala";
        BB = textfield(R, s, align_center);
        TEST_ASSERT(  BB.w > 0 && BB.h > 0 );
        BB = textfield(R, NULL, align_center);
        TEST_ASSERT(  BB.w == 0 && BB.h == 0 );

        cursor_setPosition(C);
    }


    TEST(text_field, uarray16_stretchDistance)
    {
        uint16_t cell0[] = {1, 1, 1};
        TEST_ASSERT_EQUAL( EXIT_FAILURE, uarray16_stretchDistance(NULL, 3, 10) );
        TEST_ASSERT_EQUAL( EXIT_FAILURE, uarray16_stretchDistance(cell0, 0, 10) );

        TEST_ASSERT_EQUAL( EXIT_SUCCESS, uarray16_stretchDistance(cell0, 3, 10) );
        TEST_ASSERT_EQUAL( 3, cell0[0] );
        TEST_ASSERT_EQUAL( 3, cell0[1] );
        TEST_ASSERT_EQUAL( 4, cell0[2] );

        TEST_ASSERT_EQUAL( EXIT_SUCCESS, uarray16_stretchDistance(cell0, 3, 0) );
        TEST_ASSERT_EQUAL( 0, cell0[0] );
        TEST_ASSERT_EQUAL( 0, cell0[1] );
        TEST_ASSERT_EQUAL( 0, cell0[2] );

        uint16_t cell1[] = {0, 0, 0};
        TEST_ASSERT_EQUAL( EXIT_SUCCESS, uarray16_stretchDistance(cell1, 3, 10) );
        TEST_ASSERT_EQUAL( 0, cell1[0]);
        TEST_ASSERT_EQUAL( 0, cell1[1]);
        TEST_ASSERT_EQUAL( 0, cell1[2]);

        uint16_t cell2[] = {2, 0, 2};
        TEST_ASSERT_EQUAL( EXIT_SUCCESS, uarray16_stretchDistance(cell2, 3, 10) );
        TEST_ASSERT_EQUAL( 5, cell2[0]);
        TEST_ASSERT_EQUAL( 0, cell2[1]);
        TEST_ASSERT_EQUAL( 5, cell2[2]);
    }


    TEST(text_field, formatString2array)
    {
        char formatstring[] = "clr";
        uint8_t align[3];

        TEST_ASSERT_EQUAL( EXIT_FAILURE, formatString2array(NULL, formatstring, 3) );
        TEST_ASSERT_EQUAL( EXIT_FAILURE, formatString2array(align, NULL, 3) );
        TEST_ASSERT_EQUAL( EXIT_SUCCESS, formatString2array(align, formatstring, 0) );

        TEST_ASSERT_EQUAL( EXIT_SUCCESS, formatString2array(align, formatstring, 3) );
        TEST_ASSERT_EQUAL( align_center, align[0] );
        TEST_ASSERT_EQUAL( align_left, align[1] );
        TEST_ASSERT_EQUAL( align_right, align[2] );

        TEST_ASSERT_EQUAL( EXIT_SUCCESS, formatString2array(align, "ll", 2) );

        prepareErrorCondition();
        if( TEST_PROTECT() )
        {
            formatString2array(align, "llb", 3);
        }
        TEST_ASSERT_TRUE( isErrorConditionTriggered() );
        clearErrorCondition();
    }




    TEST(text_field, textarray)
    {
        Rect_t R = gfx_getDisplayDimensions();
        uint8_t format[] = {align_center, align_left};
        char outputstring[] = "Hallo\tDu";
        uint16_t width[] = {1, 2, 3};
        TEST_ASSERT_EQUAL( EXIT_FAILURE, textarray(R, outputstring, format, width, 0) );
        TEST_ASSERT_EQUAL( EXIT_FAILURE, textarray(R, outputstring, format, NULL, 2) );
        TEST_ASSERT_EQUAL( EXIT_FAILURE, textarray(R, outputstring, NULL, width, 2) );
        TEST_ASSERT_EQUAL( EXIT_FAILURE, textarray(R, NULL, format, width, 2) );
        TEST_ASSERT_EQUAL( EXIT_SUCCESS, textarray(R, outputstring, format, width, 2) );

        TEST_ASSERT_EQUAL( EXIT_SUCCESS, textarray(R, outputstring, format, width, 3) );


    }







#else
    typedef int this_is_no_empty_translation_unit;

#endif

