/*
 * $Date: 2020-08-01 09:01:46 +0200 (Sa, 01 Aug 2020) $
 * $Revision: 27 $
 */


#include "sw_config.h"



#if defined( INCLUDE_UNIT_TESTS )

    #include "tft_text.h"
    #include "unity_fixture.h"
    #include "font.h"
    #include "sw_config.h"
    #include "text_window.h"
    #include "u8g2_fontdata.h"


    Textwindowdump_t Dump;

    TEST_GROUP(tft_text);
    TEST_SETUP(tft_text)
    {
        // Textfont sichern, damit bei Modultests auf der Zielplattform nicht die komplette
        // Ausgabe geschrottet wird.
        Dump =  textwindow_dumpSettings();
        textwindow_setFont(font_largestFontAvailable);
    }


    TEST_TEAR_DOWN(tft_text)
    {
        textwindow_restoreSettings(Dump);
    }

    TEST_GROUP_RUNNER(tft_text)
    {
        RUN_TEST_CASE(tft_text, strBoundingBox_monospaced);
        RUN_TEST_CASE(tft_text, font_lineBbx);
    }



    TEST(tft_text, strBoundingBox_monospaced)
    {
        //TEST_IGNORE();
        const uint8_t *font = font_monospaceTerminal;
        assert( font != NULL );
        //textwindow_setFont(font_monospaceTerminal);
        int8_t xOffset = -1;
        int8_t yOffset = -8;
        int8_t width = 7;
        int8_t height = 11;

        // Die Breiten-Werte sind ausprobiert, damit der Test nicht mehr fehlschlaegt
        // Genau stimmt die Bounding-Box nicht, aber fuer die Anwendung reicht es bislang

        Rect_t R = font_lineBbx(font, "");
        TEST_ASSERT_EQUAL( xOffset, R.x );
        TEST_ASSERT_EQUAL( yOffset, R.y );
        TEST_ASSERT_EQUAL( 0, R.w );
        TEST_ASSERT_EQUAL( height, R.h );

        R = font_lineBbx(font, "lala");
        TEST_ASSERT_EQUAL( xOffset, R.x );
        TEST_ASSERT_EQUAL( yOffset, R.y );
        TEST_ASSERT_EQUAL( 25, R.w );        // Ausprobiert
        TEST_ASSERT_EQUAL( height, R.h );

        R = font_lineBbx(font, "la\nla");
        TEST_ASSERT_EQUAL( xOffset, R.x );
        TEST_ASSERT_EQUAL( yOffset, R.y );
        TEST_ASSERT_EQUAL( 2*width+xOffset, R.w );
        TEST_ASSERT_EQUAL( height, R.h );

        R = font_lineBbx(font, "lai\tla");
        TEST_ASSERT_EQUAL( xOffset, R.x );
        TEST_ASSERT_EQUAL( yOffset, R.y );
        TEST_ASSERT_EQUAL( 19, R.w ); // Ausprobiert
        TEST_ASSERT_EQUAL( height, R.h );
    }


    TEST(tft_text, font_lineBbx)
    {
        // Die Breiten-Werte sind ausprobiert, damit der Test nicht mehr fehlschlaegt
        // Genau stimmt die Bounding-Box nicht, aber fuer die Anwendung reicht es bislang

        //textwindow_setFont(font_breadAndButter);
        const uint8_t *font = font_breadAndButter;

        Rect_t R = font_lineBbx(font, "");
        //TEST_ASSERT_EQUAL( 0, R.x );
        //TEST_ASSERT_EQUAL( 0, R.y );
        TEST_ASSERT_EQUAL( 0, R.w );
        TEST_ASSERT_EQUAL( 20, R.h );

        R = font_lineBbx(font, "lala");
        //TEST_ASSERT_EQUAL( 0, R.x );
        //TEST_ASSERT_EQUAL( 0, R.y );
        TEST_ASSERT_EQUAL( 27, R.w );
        TEST_ASSERT_EQUAL( 20, R.h );

        R = font_lineBbx(font, "la\nla");
        //TEST_ASSERT_EQUAL( 0, R.x );
        //TEST_ASSERT_EQUAL( 0, R.y );
        TEST_ASSERT_EQUAL( 14, R.w );
        TEST_ASSERT_EQUAL( 20, R.h );

        R = font_lineBbx(font, "la\tla");
        //TEST_ASSERT_EQUAL( 0, R.x );
        //TEST_ASSERT_EQUAL( 0, R.y );
        TEST_ASSERT_EQUAL( 14, R.w );
        TEST_ASSERT_EQUAL( 20, R.h );
    }



#else
    typedef int this_is_no_empty_translation_unit;

#endif
