/** @file text_cursor.h
 *
 * Textcursor lesen, schreiben und manipulieren
 *
 * Nahezu keine dieser Funktionen ist reentrant.
 *
 * $Date: 2020-06-17 11:20:09 +0200 (Mi, 17 Jun 2020) $
 * $Revision: 12 $
 *
 * 07.03.2018 Nicolas */

#ifndef TEXT_CURSOR_H_INCLUDED
    #define TEXT_CURSOR_H_INCLUDED

    #include "coord.h"
    #include "rect.h"
    #include "text_window.h" /* Textwindow_t */


    #ifndef static_inline
        #define static_inline static inline __attribute__((always_inline))
    #endif


    /* Das globale Textfenster-Struct enthaelt alle Cursordaten */
    extern Textwindow_t Textwindow;


    /** GET-Funktion: Zeiger auf aktuelle Positionsdaten */
    static_inline Coord2_t * cursor_getPtr(void)
    {
        return &(Textwindow.Cursor);
    }



    /** GET-Funktion: Aktuelle Positionsdaten */
    static_inline Coord2_t cursor_getPosition(void)
    {
        return *(cursor_getPtr());
    }



    /** GET-Funktion: Zeiger auf Elternfenster des Cursors */
    static_inline Rect_t * cursor_getWindowPtr(void)
    {
        return &(Textwindow.Window);
    }



    /** SET-Funktion: Cursor-Positionsdaten
     * @param[in] Pos: Neue Position */
    static_inline void cursor_setPosition(Coord2_t Pos)
    {
        Coord2_t *Cursor = cursor_getPtr();
        *Cursor = Pos;
    }



    /** Cursor an die obere linke Ecke des Textfensters setzen */
    static_inline void cursor_home(void)
    {
        Rect_t R = text_getWindow();
        cursor_setPosition((Coord2_t) {R.x, R.y} );
    }



    /** Zeilenumbruch vornehmen */
    static_inline void cursor_newline(void)
    {
        Rect_t *Window = cursor_getWindowPtr();
        Coord2_t *Cursor = cursor_getPtr();
        int_fast16_t lineheight = window_getLineHeight();

        Cursor->x = Window->x;
        Cursor->y += lineheight;

        /* Umbruch unterhalb Bildschirm */
        if( Cursor->y + lineheight > Window->y + Window->h )
        {
            Cursor->y = Window->y;
        }
    }



    /** Zeilenvorschub vor dem Zeichen.
     * wenn das Zeichen nicht mehr in die aktuelle Zeile passt, wird ein Zeilenumbruch
     * vorgenommen. Weil ueblicherweise anschliessend die Cursorposition direkt fuer die Darstellung
     * eines Zeichens benoetigt wird, wird der aktuelle Wert direkt als Zeiger zurueckgegeben.
     * @param[in] charWidth: Zeichenbreite
     * @return Zeiger auf aktuelle Cursorposition */
    static inline Coord2_t cursor_linebreakBeforeChar(ifast16_t charWidth)
    {
        Rect_t *Window = cursor_getWindowPtr();
        Coord2_t *Cursor = cursor_getPtr();

        /* Links oben einpassen */
        if( (Cursor->x < Window->x) )
        {
            Cursor->x = Window->x;
        }
        if( (Cursor->y < Window->y) )
        {
            Cursor->y = Window->y;
        }


        /* Zeilenumbruch */
        ifast16_t rightLimit = (Window->x + Window->w);
        if( (Cursor->x + charWidth) > rightLimit )
        {
            cursor_newline();
        }

        return *Cursor;
    }



    /** Zeichenvorschub nach dem Zeichen.
     * Der Cursor kann nach einem Zeichen durchaus ausserhalb des Textfeldes stehen. Vor
     * dem naechsten Zeichen wird er wieder passend geschoben.
     * @param[in] pitch: Zeichenvorschub
     * @return EXIT_SUCCESS */
    static inline exit_t cursor_advanceAfterChar(ifast16_t pitch)
    {
        Coord2_t *Cursor = cursor_getPtr();

        Cursor->x += pitch;

        return EXIT_SUCCESS;
    }



#endif /* TEXT_CURSOR_H_INCLUDED */
