/*
 * $Date: 2020-08-11 18:40:35 +0200 (Di, 11 Aug 2020) $
 * $Revision: 58 $
 */

#include "sw_config.h"



#if defined( INCLUDE_UNIT_TESTS )

    #include "font.h"
    #include "u8g2_font_find_glyph.h"
    #include "sw_config.h"
    #include "u8g2_fontdata_icons.h"
    #include "u8g2_font_decoder.h"
    #include "u8g2_font_header_private.h"
    #include "tft_text.h"
    #include "unity_fixture.h"
    #include "debughelp.h"


    TEST_GROUP(fontU8g2);
    TEST_SETUP(fontU8g2)
    {

    }

    TEST_TEAR_DOWN(fontU8g2)
    {

    }

    TEST_GROUP_RUNNER(fontU8g2)
    {
        RUN_TEST_CASE(fontU8g2, fontheader);
        RUN_TEST_CASE(fontU8g2, fontinfo);

        RUN_TEST_CASE(fontU8g2, fontboundingboxPlausibilityMono);
        RUN_TEST_CASE(fontU8g2, fontboundingboxPlausibilityProp);
        RUN_TEST_CASE(fontU8g2, invalidChar);
        RUN_TEST_CASE(fontU8g2, findNextChar);
        RUN_TEST_CASE(fontU8g2, findNextCharOver255);
    }



    TEST(fontU8g2, fontheader)
    {
        const uint8_t *font = font_test;
        TEST_ASSERT_EQUAL( -4,  (int8_t) pgm_read_byte(font + u8g2_font_header_offset_y_offset) );
        TEST_ASSERT_EQUAL( -4, u8g2_font_getBoundingBox(font).y );
    }



    TEST(fontU8g2, getFontBoundingBox)
    {

        u8g2_font_getBoundingBox( NULL );
    }



    TEST(fontU8g2, fontinfo)
    {
        const uint8_t *const font = font_test;
        TEST_ASSERT_NOT_NULL( font );
        TEST_ASSERT_GREATER_OR_EQUAL( 127-32, u8g2_font_getNumberOfGlyphs(font) );
        TEST_ASSERT_LESS_OR_EQUAL( 255, u8g2_font_getNumberOfGlyphs(font) );
        uint16_t upperA = u8g2_font_getStartPosUpperA(font);
        uint16_t lowerA = u8g2_font_getStartPosLowerA(font);
        uint16_t unicode = u8g2_font_getStartPosUnicode(font);
        TEST_ASSERT_GREATER_THAN( upperA, lowerA );
        TEST_ASSERT_GREATER_THAN( lowerA, unicode );
    }



    void calcFontBB(const uint8_t *font, char *fontname)
    {
        TEST_ASSERT( font != NULL );
        TEST_ASSERT( fontname != NULL );

        //printf("\n font=%s\n", fontname);
        // Calculate Font Boundig box as outline of all glyph bounding boxes
        uint16_t unicode = 0;
        Rect8_t FbbCalc = {0, 0, 0, 0};

        while( (unicode = u8g2_font_findNextChar(font, unicode)) )
        {
            Rect8_t Gbb = font_glyphOutline(font, unicode).BBx;
            FbbCalc = rect8_outline(FbbCalc, Gbb);
        }

        // Get font bounding box as part of font data
        Rect8_t FbbSaved = u8g2_font_getBoundingBox(font);

        //printf(" Font bounding box (font): ");
        //rect_print(rect8ToRect(FbbSaved));

        //printf(" Font bounding box (calculated): ");
        //rect_print(rect8ToRect(FbbCalc));

        TEST_ASSERT_EQUAL( FbbSaved.x, FbbCalc.x );
        TEST_ASSERT_EQUAL( FbbSaved.y, FbbCalc.y );
        TEST_ASSERT_EQUAL( FbbSaved.w, FbbCalc.w );
        TEST_ASSERT_EQUAL( FbbSaved.h, FbbCalc.h );
    }


    #define CALCFONTBB(FONT) (calcFontBB(FONT, #FONT))


    TEST(fontU8g2, fontboundingboxPlausibilityMono)
    {
        CALCFONTBB(font_monospaceTerminal);
    }



    TEST(fontU8g2, fontboundingboxPlausibilityProp)
    {
        CALCFONTBB(font_largestFontAvailable);
    }



    TEST(fontU8g2, invalidChar)
    {
        const uint8_t *const font = font_test;
        TEST_ASSERT_EQUAL( NULL, u8g2_font_findCharacterGlyphPtr(font, 0) );
    }



    TEST(fontU8g2, findNextChar)
    {
        const uint8_t *const font = font_test;
        TEST_ASSERT_EQUAL( 32, u8g2_font_findNextChar(font, 0) );
        TEST_ASSERT_EQUAL( 'b', u8g2_font_findNextChar(font, 'a') );
        TEST_ASSERT_EQUAL( 0 , u8g2_font_findNextChar(font, INT16_MAX) );
    }



    TEST(fontU8g2, findNextCharOver255)
    {
        const uint8_t * font = open_iconic_selection_4x;
        uint16_t expected[] = {66, 67, 80, 82, 87, 102, 120, 121, 129, 188, 197, 202, 203, 204,
            214, 243, 280, 281, 282, 300, 301, 302, 303, 304, 305, 306, 307, 308, 309, 310, 311,
            312, 313, 314, 315, 316, 317, 319, 320, 321, 322, 323, 324, 325};
        uint16_t unicode = 0;

        for( unsigned int i = 0; i < sizeof(expected)/sizeof(expected[0]); i++ )
        {
            TEST_ASSERT_EQUAL( expected[i], unicode = u8g2_font_findNextChar(font, unicode));
        }
    }

#else
    typedef int this_is_no_empty_translation_unit;

#endif
