/** @file font.h
 *
 * Allgemeine (austauschbare) Funktionen zu Zeichensatz und Zeichendarstellung
 *
 * $Date: 2020-08-02 19:08:25 +0200 (So, 02 Aug 2020) $
 * $Revision: 32 $
 *
 * @date 08.03.2018 @author Nicolas */


/**
 @defgroup Font_Subsystem
 */


#ifndef FONT_H_INCLUDED
    #define FONT_H_INCLUDED


    #include <stdint.h>
    #include <stdbool.h>
    #include "rect.h"
    #include "color.h"
    #include "u8g2_font_find_glyph.h"


    enum font_codepage_e
    {
        CP_COMPACT = 0, /**< kurze Codepage */
        CP_MSDOS   = 1, /**< Codepage 850 */
        CP_UNICODE16,
    };



    /** Glyph-Bounding-Box-Modus */
    enum glyph_bbx_mode_e
    {
        bbx_mode_tight,  /**< Bounding Box enganliegend, kein Offset */
        bbx_mode_line,   /**< Ganze Zeile, wie im Bleisatz, x-Offset */
        bbx_mode_block,  /**< Bounding-Box ueber Fontbreite (monospaced) */
    };



    /** Glyph header data
     * Die Bestimmung der Glyph-Bounding-Box bedingt die Suche nach der Glyph-Adresse und eine
     * komplette Dekodierung des headers. Deswegen wird die Glyph-Adresse vorgehalten, um die
     * Dekodierung der Bitmap zu beschleunigen. */
    typedef struct GlyphOutline_s
    {
        Rect8_t BBx;                    /**< Glyph bounding box */
        int8_t pitch;                   /**< Zeichenvorschub */
    }
    GlyphOutline_t;



    void font_drawGlyph(Coord2_t Pos, const uint8_t *font, uint16_t unicode,
                                  Colors_t Color, enum glyph_bbx_mode_e bbxMode);


    Rect_t font_fontBbx(const uint8_t *font);
    bool font_glyphExists(const uint8_t *font, uint16_t unicode);

    const uint8_t * font_glyph(const uint8_t *font, uint16_t unicode);
    GlyphOutline_t font_glyphOutline(const uint8_t *font, uint16_t unicode);
    GlyphOutline_t font_ersatzOutline(const uint8_t *font);



    /** Find unicode of next available character/glyph.
     *
     * If no following character exists, return value = 0
     *
     * @param[in] font: Font-Array
     * @param[in] unicode: 16-Bit-Unicode des Glyphen
     * @return 16-Bit-Unicode des nachfolgenden Glyphen oder 0 */
    static inline uint16_t font_findNextChar(const uint8_t *font, uint16_t unicode)
    {
        return u8g2_font_findNextChar(font, unicode);
    }


    Rect_t font_drawGlyphAligned(Rect_t R, const uint8_t *font, uint16_t unicode, Colors_t Color,
                         enum rect_orientation_e orientation);

#endif
